
#!/usr/bin/env python

import os
from string import find

from MolKit import Read
from MolKit.pdbWriter import PdbqWriter
from MolKit.molecule import AtomSet

from AutoDockTools.MoleculePreparation import AD4LigandPreparation


if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: repair_ligand4.py -s filename"
        print
        print "    Description of command..."
        print "        [-f]    pdbqt_filename"
        print "    Optional parameters:"
        print "        [-o]    alternative pdbqt_filename"
        print "        (default is 'repaired_' +pdbqt_filename)"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'f:o:v')

    except getopt.GetoptError, msg:
        print 'repair_ligand4.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-f: pdbqt_filename_stem
    pdbqt_filename =  None

    # optional parameters
    verbose = None
    outputfilename =  None

    #'f:o:v'
    for o, a in opt_list:
        if o in ('-f', '--f'):
            pdbqt_filename = a
            if verbose: print 'set pdbqt_filename to ', pdbqt_filename
            outputfilename =  'repaired_' + pdbqt_filename
        if o in ('-o', '--o'):
            outputfilename = a 
            if verbose: 
                print 'set output outputfilename to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()


    if not pdbqt_filename:
        print 'repair_ligand4: pdbqt_filename must be specified.'
        usage()
        sys.exit()

    mol = Read(pdbqt_filename)[0]
    if verbose: print 'read ', pdbqt_filename
    mol.buildBondsByDistance()
    mol.LPO = AD4LigandPreparation(mol, mode='interactive', repairs='', charges_to_add=None,
                root=0, outputfilename=outputfilename, cleanup='')
    #rebuild torTree
    allAts = mol.allAtoms
    for b in allAts.bonds[0]:
        if b.activeTors: b.activeTors = 0
    torscount = 0
    tM = mol.torTree.torsionMap
    for i in range(len(tM)):
        bnum0, bnum1 = tM[i].bond
        a0 = allAts.get(lambda x: x.number==bnum0 + 1)[0]
        a0.tt_ind = bnum0
        a1 = allAts.get(lambda x: x.number==bnum1 + 1)[0]
        a1.tt_ind = bnum1
        b = AtomSet([a0,a1]).bonds[0]
        if hasattr(b, 'possibleTors'):
            assert b.possibleTors
        else:
            b.possibleTors = 1
        b.activeTors = 1
        torscount = torscount + 1
    mol.torscount = torscount
    mol.ROOT = mol.allAtoms[0]
    mol.ROOT.rnum0 = 0
    mol.LPO.write(outputfilename)


# To execute this command type:
# repair_ligand4.py -f pdbqt_filename [-o outputfilename] -v







#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/summarize_results.py,v 1.5 2009/06/01 17:35:00 rhuey Exp $
#
import os, glob

from MolKit import Read
from MolKit.hydrogenBondBuilder import HydrogenBondBuilder
from AutoDockTools.Docking import Docking
from mglutil.math.rmsd import RMSDCalculator
from PyAutoDock.InternalEnergy import InternalEnergy
from PyAutoDock.MolecularSystem import MolecularSystem
from PyAutoDock.AutoDockScorer import AutoDock305Scorer

def get_filename(d, conf):
    #find the filename of dlg containing this conformation
    for k in d.dlo_list:
        if conf in k.conformations:
            dlo_filename = os.path.splitext(k.parser.filename)[0]
            break
    return dlo_filename


def get_energy_breakdown(receptor_atoms, ligand_atoms, coords_to_use):
    ms = MolecularSystem()
    ms.add_entities(receptor_atoms)
    ms.add_entities(ligand_atoms)
    ms.set_coords(1, coords_to_use) 
    #ms.set_coords(1, c.getCoords()[:])
    ad_scorer = AutoDock305Scorer()
    ad_scorer.set_molecular_system(ms)
    score_list = ad_scorer.get_score_per_term()
    ostr = ""
    for score in score_list:
        ostr = ostr + "% 8.4f," %score
    return ostr


def construct_hydrogen_bonds(receptor_atoms, ligand_atoms):
    atDict = HydrogenBondBuilder().build(receptor_atoms, ligand_atoms)
    num_hbonds = len(atDict)
    ostr = "%2d," % num_hbonds
    return ostr


def get_best_energy_info(d, rms_tolerance, build_hydrogen_bonds=False, \
                                report_energy_breakdown=False,
                                receptor_filename=None, refCoords=None):
    ostr = "" 
    clust0 = d.clusterer.clustering_dict[rms_tolerance][0]
    c = clust0[0]
    #find the filename of the best result
    dlo_filename = get_filename(d, c)
    if verbose: print "set dlo_filename to ", dlo_filename
    ostr = dlo_filename + ", "
    de = c.docking_energy
    if refCoords:
        ostr += "%3d,%3d,%3d,% 8.4f,% 8.4f," %(len(d.clusterer.data), num_clusters, len(clust0),  de, c.getRMSD(refCoords=refCoords))
    else:
        ostr += "%3d,%3d,%3d,% 8.4f,% 8.4f," %(len(d.clusterer.data), num_clusters, len(clust0),  de, c.getRMSD())
    d.ch.set_conformation(c)
    receptor = None
    if build_hydrogen_bonds:
        if not receptor_filename:
            print "receptor_filename must be specified in order to build_hydrogen_bonds"
            return 
        receptor = Read(receptor_filename)[0] 
        receptor.buildBondsByDistance()
        d.ligMol.buildBondsByDistance()
        ostr += construct_hydrogen_bonds(receptor.allAtoms, d.ligMol.allAtoms)
    if report_energy_breakdown:
        if not receptor_filename:
            print "receptor_filename must be specified in order to build_hydrogen_bonds"
        if not receptor:
            receptor = Read(receptor_filename)[0] 
            receptor.buildBondsByDistance()
        #build bonds if necessary 
        d.ligMol.buildBondsByDistance()
        #ostr += get_energy_breakdown(receptor.allAtoms, d.ligMol.allAtoms, c)
        ostr += get_energy_breakdown(receptor.allAtoms, d.ligMol.allAtoms, c.getCoords()[:])

    return ostr


def get_largest_cluster_info(d, rms_tolerance, build_hydrogen_bonds=False, \
                                    report_energy_breakdown=False,
                                    receptor_filename=None, refCoords=None):
    largest = d.clusterer.clustering_dict[rms_tolerance][0]
    if verbose: print "set largest to ", len(largest)
    for clust in d.clusterer.clustering_dict[rms_tolerance]:
        if verbose: print "current largest cluster len= ", len(clust)
        if len(clust)>len(largest): 
            if verbose: print "resetting largest clust: now len=", len(clust)
            largest = clust
    c = largest[0]   #print info about the lowest energy member of this cluster
    ostr = " "
    dlo_filename = get_filename(d, c)
    ostr += dlo_filename + ","
    de = c.docking_energy
    if refCoords:
        ostr += "%3d,%3d,%3d,% 8.4f,% 8.4f," %(len(d.clusterer.data), num_clusters, len(largest),  de, c.getRMSD(refCoords=refCoords))
    else:
        ostr += "%3d,%3d,%3d,% 8.4f,% 8.4f," %(len(d.clusterer.data), num_clusters, len(largest),  de, c.getRMSD())
    if verbose: print "set dlo_filename to ", dlo_filename
    #update the coords only if you need to?
    d.ch.set_conformation(c)
    receptor = None
    if build_hydrogen_bonds:
        if not receptor_filename:
            print "receptor_filename must be specified in order to build_hydrogen_bonds"
            return 
        receptor = Read(receptor_filename)[0] 
        receptor.buildBondsByDistance()
        ostr += construct_hydrogen_bonds(receptor.allAtoms, d.ligMol.allAtoms)
    if report_energy_breakdown:
        if not receptor_filename:
            print "receptor_filename must be specified in order to build_hydrogen_bonds"
            return 
        if not receptor:
            receptor = Read(receptor_filename)[0] 
            receptor.buildBondsByDistance()
        #ostr += get_energy_breakdown(receptor.allAtoms, d.ligMol.allAtoms, c)
        ostr += get_energy_breakdown(receptor.allAtoms, d.ligMol.allAtoms, c.getCoords()[:]) 
    return ostr


if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: summarize_results.py -d directory"
        print
        print "    Description of command..."
        print "         -d     directory"
        print "    Optional parameters:"
        print "        [-t]    rmsd tolerance (default is 1.0)"
        print "        [-f]    rmsd reference filename "
        print "        (default is to use input ligand coordinates from docking log)"
        print "        [-b]    print best docking info only (default is print all)"
        print "        [-L]    print largest cluster info only (default is print all)"
        print "        [-B]    print best docking and largest cluster info only (default is print all)"
        print "        [-o]    output filename"
        print "                      (default is 'summary_of_results')"
        print "        [-a]    append to  output filename"
        print "                      (default is to open output filename 'w')"
        print "        [-k]    build hydrogen bonds"
        print "        [-r]    receptor filename"
        print "        [-e]    report energy breakdown"
        print "        [-v]    verbose output"
        print "                      (default is leave active)"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'd:o:t:f:r:bLBkaevh')
    except getopt.GetoptError, msg:
        print 'summarize_results.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-d: directory
    directory =  None
    #-t: rms_tolerance
    rms_tolerance =  1.0
    #-f: rms reference
    rms_reference = None
    # optional parameters
    verbose = None
    #-o outputfilename
    outputfilename = "summary_of_results"
    #-r receptor_filename
    receptor_filename = None
    #-b print best only
    print_best_only = False
    #-L print Largest cluster only
    print_Largest_only = False
    #-B print best and Largest cluster only
    print_best_and_Largest_only = False
    #-k build hydrogen bonds 
    build_hydrogen_bonds = False
    #-a append to outputfile
    append_to_outputfile = False
    #-e report_energy_breakdown 
    report_energy_breakdown = False


    #'d:o:t:r:f:bLBkaevh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-d', '--d'):
            directory = a
            if verbose: print 'set directory to ', a
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-t', '--t'):
            rms_tolerance = float(a)
            if verbose: print 'set rms_tolerance to ', a
        if o in ('-f', '--f'):
            rms_reference = a
            if verbose: print 'set rms_reference to ', a
        if o in ('-r', '--r'):
            receptor_filename = a
            if verbose: print 'set receptor_filename to ', a
        if o in ('-b', '--b'):
            print_best_only = True
            if verbose: print 'set print_best_only to ', True
        if o in ('-L', '--L'):
            print_Largest_only = True
            if verbose: print 'set print_Largest_only to ', True
        if o in ('-B', '--B'):
            print_best_and_Largest_only = True
            print_best_only = True
            print_Largest_only = True
            if verbose: print 'set print_best_and_Largest_only to ', True
        if o in ('-k', '--k'):
            build_hydrogen_bonds = True
            if verbose: print 'set build_hydrogen_bonds to ', True
        if o in ('-a', '--a'):
            append_to_outputfile = True
            if verbose: print 'set append_to_outputfile to ', True
        if o in ('-e', '--e'):
            report_energy_breakdown = True
            if verbose: print 'set report_energy_breakdown to ', True
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    if outputfilename=='summary_of_results':
        outputfilename = outputfilename + '_' + str(rms_tolerance)
    if build_hydrogen_bonds is True and receptor_filename is None:
        print 'to build hydrogen bonds, receptor filename must be specified'
        sys.exit()
    if report_energy_breakdown is True and receptor_filename is None:
        print 'to report energy breakdown, receptor filename must be specified'
        sys.exit()
    if not  directory:
        print 'summarize_results: directory must be specified.'
        usage()
        sys.exit()

    #read all the docking logs in as one Docking
    dlg_list = glob.glob(directory + '/*.dlg')
    d = Docking()
    for dlg in dlg_list:
        d.readDlg(dlg)

    #setup rmsd tool
    coords = d.ligMol.allAtoms.coords[:]
    refCoords = None
    if rms_reference is not None:
        file = directory + '/' + rms_reference
        ref = Read(file)
        if not len(ref): 
            print "unable to read ", rms_reference
        else:
            ref = ref[0]
            coords = ref.allAtoms.coords
            refCoords = coords[:]
    d.clusterer.rmsTool = RMSDCalculator(coords)

    d.clusterer.make_clustering(rms_tolerance) 

    # for building hydrogen bonds or reporting energy breakdown
    # setup receptor:
    if build_hydrogen_bonds or report_energy_breakdown:
        d.ligMol.buildBondsByDistance()
        receptor_filename = directory + '/' + receptor_filename
        receptor = Read(receptor_filename)[0]
        receptor.buildBondsByDistance()
    if build_hydrogen_bonds:
        hbondBuilder = HydrogenBondBuilder()
        #do next two lines for each conf
        #d.ligMol.set_conformation(conf)
        #atDict = hbondBuilder.build(receptor.allAtoms, d.ligMol.allAtoms)
        #num_hbonds= len(atDict)
    if report_energy_breakdown:
        ms = MolecularSystem()
        ms.add_entities(receptor.allAtoms)
        ms.add_entities(d.ligMol.allAtoms)
        ad305scorer = AutoDock305Scorer()
        ad305scorer.set_molecular_system(ms)
        #whenever the ligMol changes conf, 
        #need to call ms.set_coords(1, new_coords)

    mode = 'w'
    first = True
    if append_to_outputfile:
        mode = 'a'
        first = not os.path.exists(outputfilename)
    fptr = open(outputfilename, mode)
    num_clusters = len(d.clusterer.clustering_dict[rms_tolerance])
    #output a header if file opened with mode 'w' or if this is first use of outputfile...
    if verbose: print "first is ", first
    if first:
        tstr = ""
        if print_best_only:
            tstr += "lowestEnergy_dlgfn     #runs #cl #LEC    LE      rmsd_LE "
            if build_hydrogen_bonds:
                tstr+= " #hb "
            if report_energy_breakdown:
                tstr+= " #ESTAT      #HB     #VDW   #DSOLV "
        if print_Largest_only:
            if print_best_only:
                tstr += "   "
            tstr += " largestCl_dlgfn    #runs #cl #LC    LE_LC     rmsd_LC "
            if build_hydrogen_bonds:
                tstr+= " #hb "
            if report_energy_breakdown:
                tstr+= " #ESTAT      #HB     #VDW   #DSOLV "
        if not print_best_only and not print_Largest_only:
            tstr = "#dlgfn                      #in cluster #LE   #rmsd "
        tstr+= "#ats #tors"
        tstr+="\n"
        fptr.write(tstr)
        first = False
    ostr = ""
    if print_best_only or print_Largest_only:
        if print_best_only:
            if verbose: print 'print_best_only is True'
            ostr += get_best_energy_info(d, rms_tolerance, build_hydrogen_bonds, report_energy_breakdown,  receptor_filename, refCoords=refCoords)
        if print_Largest_only:
            if verbose: print 'print_Largest_only is True'
            ostr += get_largest_cluster_info(d, rms_tolerance, build_hydrogen_bonds, report_energy_breakdown, receptor_filename, refCoords=refCoords)
        #add number of atoms and number of torsions to output string
        ostr += "%4d,%2d" %(len(d.ligMol.allAtoms), d.ligMol.parser.keys.count('BRANCH'))
        ostr +='\n'
        try:
            fptr.write(ostr)
        except:
            print 'except'
            fptr.write("\n")
    else:
        receptor = None
        if receptor_filename is not None:
            receptor = Read(receptor_filename)
            receptor = receptor[0]
            receptor.buildBondsByDistance()
        for clust in d.clusterer.clustering_dict[rms_tolerance]:
            if verbose:
                print "len(d.clusterer.clustering_dict)=", len(d.clusterer.clustering_dict[rms_tolerance])
                print "len(clust)=", len(clust)
            #for each cluster in the clustering at this rms_tolerance:
            #output length of cluster and energies of the lowest energy result
            c = clust[0]
            if verbose: print "setting c to clust[0]", id(c)
            #write the dlg filename for the best in this cluster
            dlo_filename = get_filename(d, c)
            ostr = dlo_filename + ", " 
            fptr.write(ostr)
            if build_hydrogen_bonds or report_energy_breakdown:
                if verbose: print "setting conf to ", id(c)
                d.ch.set_conformation(c)
            de = c.docking_energy
            ostr = "%3d,% 8.4f,% 8.4f" %(len(clust), de, c.getRMSD(refCoords=refCoords))
            if build_hydrogen_bonds and receptor:
                ostr += construct_hydrogen_bonds(receptor.allAtoms, d.ligMol.allAtoms)
            if report_energy_breakdown and receptor:
                #ostr += get_energy_breakdown(receptor.allAtoms, d.ligMol.allAtoms, c)
                ostr += get_energy_breakdown(receptor.allAtoms, d.ligMol.allAtoms, c.getCoords()[:])
            ostr += "%4d,%3d" %(len(d.ligMol.allAtoms), d.ligMol.parser.keys.count('BRANCH'))
            ostr = ostr + "\n"
            fptr.write(ostr)
    fptr.close()
    

# To execute this command type:
# summarize_results.py -d directory -t rmsd tolerance -r receptor filename  
#                         -b report best_docking_only 
#                         -k build_hydrogen_bonds 
#                         -f rms reference filename
#                         -L report largest cluster information 
#                         -B print both best docking info  and largest cluster info
#                         -e report_energy_breakdown 
#                         -v verbose
# NOTE: -d, -t, -r and -f require arguments whereas the others set booleans




#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/write_clustering_histogram_postscript.py,v 1.3 2007/10/08 18:14:58 rhuey Exp $
#
# $Id: write_clustering_histogram_postscript.py,v 1.3 2007/10/08 18:14:58 rhuey Exp $
#
import os, glob, Tkinter, numpy.oldnumeric as Numeric
from MolKit import Read

from AutoDockTools.Docking import Docking
from AutoDockTools.interactiveHistogramGraph import InteractiveHistogramGraph
from AutoDockTools.histogram import HistogramRI
from mglutil.math.rmsd import RMSDCalculator




if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: write_clustering_histogram_postscript.py -d directory"
        print
        print "    Description of command..."
        print "         -d     directory"
        print "    Optional parameters:"
        print "        [-t]    rmsd tolerance (default is 1.0)"
        print "        [-o]    output filename"
        print "                      (default is 'directory.ps')"
        print "        [-v]    verbose output"
        print "                      (default is leave active)"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'd:o:t:vh')
    except getopt.GetoptError, msg:
        print 'write_clustering_histogram_postscript.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-d: directory
    directory =  None
    #-t: rms_tolerance
    rms_tolerance =  1.0
    # optional parameters
    verbose = None
    #-o outputfilename
    outputfilename = ""


    #'d:o:t:vh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-d', '--d'):
            directory = a
            if verbose: print 'set directory to ', a
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-t', '--t'):
            rms_tolerance = float(a)
            if verbose: print 'set rms_tolerance to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    print 'directory=', directory
    if directory is None:
        print 'write_clustering_histogram_postscript: directory must be specified.'
        usage()
        sys.exit()
    if outputfilename=="":
        outputfilename = directory + '_' + str(rms_tolerance)+ '.ps'

    #read all the docking logs in as one Docking
    dlg_list = glob.glob(directory + '/*.dlg')
    d = Docking()
    for dlg in dlg_list:
        d.readDlg(dlg)
        if len(d.ch.conformations)>=50:
            print 'stop reading dlgs at', len(d.ch.conformations)
            break
    mol = d.ligMol
    crds = mol.allAtoms.coords[:]
    d.clusterer.rmsTool = RMSDCalculator(crds)
    d.clusterer.rmsToolRef = '0'   # for ADT only?
    #d.clusterer_dict['ats_binding'] = d.clusterer #for ADT only??
    d.clusterer.make_clustering(rms_tolerance) 
    s = d.ch.conformations
    elist = []
    for c in s: 
        if c.docking_energy:
            elist.append(c.docking_energy)
        elif c.binding_energy: 
            elist.append(c.binding_energy)
        elif c.energy: 
            elist.append(c.energy)
    if not len(elist): 
        print 'No energies available for docking.ch.conformations'
        exit()
    r = Tkinter.Tk()
    dataList = []
    reverseList = []
    rLctr = 0
    confL = d.ch.conformations
    e = d.clusterer.energy_used
    #for l in mol.cluSEQ:
    for l in d.clusterer.clustering_dict[rms_tolerance]:
        dataList.append([l[0].energy, len(l)])
        reverseList.append(range(rLctr, rLctr+len(l)))
    mol.elist = Numeric.array(elist)
    mol.r = [Numeric.minimum.reduce(mol.elist), 
                Numeric.maximum.reduce(mol.elist)]
    mol.nbins = Tkinter.IntVar()
    mol.nbins.set(10)
    mol.min = Tkinter.StringVar()
    mol.min.set(str(mol.r[0]))
    mol.max = Tkinter.StringVar()
    mol.max.set(str(mol.r[1]))

    r = (float(mol.min.get()), float(mol.max.get()))
    mol.ehist = HistogramRI(mol.elist,mol.nbins.get(),range=r)
    mol.ehist.createReverseIndex()
    nodeList = mol.ehist.array
    tstr = mol.name + ' histogram'
    top = Tkinter.Toplevel()
    top.title(tstr)
    mol.ehist
    #top = Tkinter.Toplevel()
    xlabel = 'ENERGY'
    mol.clustNB = InteractiveHistogramGraph(mol.name,
            master=top, nodeList = dataList, reverseIndex=reverseList,
            label_text=mol.name + ':' + str(rms_tolerance) + ' rms', xlabel_text=xlabel, 
            ylabel_text='#\nC\nO\nN\nF\nO\nR\nM\nA\nT\nI\nO\nN\nS')
    mol.clustNB.draw.update()
    mol.clustNB.draw.postscript({'file':outputfilename, 'colormode':'color'})
    top.update_idletasks()

# To execute this command type:
# write_clustering_histogram_postscript.py -d directory -t rmsd tolerance -o outputfilename




#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/prepare_pdb_split_alt_confs.py,v 1.4 2012/04/26 18:32:20 rhuey Exp $
#

import os
from MolKit import Read
from MolKit.pdbParser import PdbParser

def hasAt(name):
    if '@' in name: 
        return 1
    else: 
        return 0

def getAT_types(m):
    AT_SET = m.allAtoms.get(lambda x: hasAt(x.name))
    AT_SET_SET = set(AT_SET.name)
    alt_items = {}
    for ent in AT_SET_SET:
        alt_items[ent.split("@")[1]] = 1
    print "@@returning ", alt_items.keys(), " @@"
    return alt_items.keys()
    

if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: prepare_pdb_split_alt_confs.py -r filename.pdb"
        print
        print "    Description of command..."
        print "        -r   filename.pdb "
        print "        create separate pdb files for file with alt_loc coords"
        print "    Optional parameters:"
        print "        [-n]  remove '   new' and '   flip' at end of lines (apparent issue from MolProbity)"
        print "        [-f]  filename to contain lines after new and flip removed (MolProbity)"
        print "        [-v]  verbose output (default is minimal output)"
        print "        [-o pdb_stem]  (default creates 'filename_A.pdb', 'filename_B.pdb' etc)"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'r:nf:vo:h')

    except getopt.GetoptError, msg:
        print 'prepare_pdb_split_alt_confs.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-r: filename 
    filename =  None

    # optional parameters
    #-v verbose
    verbose = None
    #-o pdb_stem
    pdb_stem = None
    #-n remove_new
    remove_new = False
    #-f filename_nonews
    filename_nonews = None

    #'r:vo:nf:h'
    for o, a in opt_list:
        if o in ('-r', '--r'):
            filename = a
            if verbose: print 'set filename to ', filename
        if o in ('-n', '--n'):
            remove_new = True
            if verbose: print 'set remove_new to ', remove_new
        if o in ('-f', '--f'):
            filename_nonews = a
            if verbose: print 'set filename for clean output ("new"+"flip" removed) to ', filename_nonews
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', verbose
        if o in ('-o', '--o'):
            pdb_stem = a
            if verbose: print 'set pdb_stem to ', pdb_stem
        if o in ('-h', '--'):
            usage()
            sys.exit()


    if not filename:
        print 'prepare_pdb_split_alt_confs: filename for cleanup must be specified.'
        usage()
        sys.exit()

    file_stem = filename.split('.')[0]
    if pdb_stem is not None:
        file_stem = pdb_stem

    if filename_nonews is None:
        filename_nonews = file_stem + "_nonews.pdb"
        if verbose: print "set filename_nonews to ", filename_nonews

    if remove_new:
        optr = open(filename)
        lines = optr.readlines()
        if verbose: print "len(lines)==", len(lines)
        new_lines = []
        for ll in lines:
            if ll.find("   new")==78:
                ll = ll[:78] + "\n"
            if ll.find("   flip")==78:
                ll = ll[:78] + "\n"
            new_lines.append(ll)
        nptr = open(filename_nonews, 'w')
        if verbose: print "len(new_lines)==", len(new_lines)
        for ll in new_lines:
            nptr.write(ll)
        nptr.close()
        filename = filename_nonews
    mols = Read(filename)
    if verbose: print 'read ', filename
    mol = mols[0]
    if len(mols)>1:
        if verbose: print "more than one molecule in file using molecule with most atoms"
        #use the molecule with the most atoms
        ctr = 1
        for m in mols[1:]:
            ctr += 1
            if len(m.allAtoms)>len(mol.allAtoms):
                mol = m
                if verbose: print "mol set to ", ctr, "th molecule with", len(mol.allAtoms), "atoms"
    ats = mol.allAtoms.get("*@*")
    if not len(ats):
        print 'Nothing to do:no alt loc atoms found in ', filename
        sys.exit()
    list_to_write = getAT_types(mol)
    ATOMLINES = mol.parser.getAtomsLines(-2,0)
    
    for altT in list_to_write:
        fn = file_stem + '_' + altT + '.pdb'
        fptr = open(fn, 'w')
        ctr = 1
        for ll in ATOMLINES:
            if ll[16]==altT: #'B'
                newL = ll[:6] +"%5d" %(ctr) + ll[11:16]+" " + ll[17:] 
                ctr = ctr + 1
            elif ll[16]==' ':
                newL = ll[:6] +"%5d" %(ctr) + ll[11:16]+" " + ll[17:] 
                ctr = ctr + 1
            else:
                newL = ""
            if len(newL): fptr.write(newL)
        fptr.close()


# To execute this command type:
# prepare_pdb_split_alt_confs.py -r pdb_file -o pdb_stem 




########################################################################
#
#    Vision Macro - Python source code - file generated by vision
#    Tuesday 25 July 2006 15:55:35 
#    
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Daniel Stoffler, Michel Sanner and TSRI
#   
# revision: Guillaume Vareille
#  
#########################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/VisionInterface/PrepareAD4Ligand.py,v 1.3 2006/07/25 23:34:02 rhuey Exp $
#
# $Id: PrepareAD4Ligand.py,v 1.3 2006/07/25 23:34:02 rhuey Exp $
#

from NetworkEditor.macros import MacroNode
class PrepareAD4Ligand(MacroNode):

    def __init__(self, constrkw={}, name='Prepare AD4 Ligands', **kw):
        kw['name'] = name
        apply( MacroNode.__init__, (self,), kw)

    def beforeAddingToNetwork(self, net):
        MacroNode.beforeAddingToNetwork(self, net)
        ## loading libraries ##
        from AutoDockTools.VisionInterface.AdtNodes import adtlib
        net.editor.addLibraryInstance(adtlib,"AutoDockTools.VisionInterface.AdtNodes", "adtlib")

        from MolKit.VisionInterface.MolKitNodes import molkitlib
        net.editor.addLibraryInstance(molkitlib,"MolKit.VisionInterface.MolKitNodes", "molkitlib")


    def afterAddingToNetwork(self):
        from NetworkEditor.macros import MacroNode
        MacroNode.afterAddingToNetwork(self)
        ## loading libraries ##
        from AutoDockTools.VisionInterface.AdtNodes import adtlib
        from MolKit.VisionInterface.MolKitNodes import molkitlib
        ## building macro network ##
        Prepare_AD4_Ligands_11 = self
        from traceback import print_exc

        ## loading libraries ##
        from AutoDockTools.VisionInterface.AdtNodes import adtlib
        self.macroNetwork.getEditor().addLibraryInstance(adtlib,"AutoDockTools.VisionInterface.AdtNodes", "adtlib")

        from MolKit.VisionInterface.MolKitNodes import molkitlib
        self.macroNetwork.getEditor().addLibraryInstance(molkitlib,"MolKit.VisionInterface.MolKitNodes", "molkitlib")

        try:
            ## saving node input Ports ##
            input_Ports_12 = self.macroNetwork.ipNode
        except:
            print "WARNING: failed to restore MacroInputNode named input Ports in network self.macroNetwork"
            print_exc()
            input_Ports_12=None

        try:
            ## saving node output Ports ##
            output_Ports_13 = self.macroNetwork.opNode
        except:
            print "WARNING: failed to restore MacroOutputNode named output Ports in network self.macroNetwork"
            print_exc()
            output_Ports_13=None

        try:
            ## saving node Manage Rotatable Bonds ##
            from AutoDockTools.VisionInterface.AdtNodes import ManageRotatableBonds
            Manage_Rotatable_Bonds_23 = ManageRotatableBonds(constrkw = {}, name='Manage Rotatable Bonds', library=adtlib)
            self.macroNetwork.addNode(Manage_Rotatable_Bonds_23,217,143)
            apply(Manage_Rotatable_Bonds_23.inputPortByName['mols'].configure, (), {'color': '#c64e70', 'cast': True, 'shape': 'oval'})
            apply(Manage_Rotatable_Bonds_23.inputPortByName['root'].configure, (), {'color': 'white', 'cast': True, 'shape': 'oval'})
            apply(Manage_Rotatable_Bonds_23.inputPortByName['allowed_bonds'].configure, (), {'color': 'white', 'cast': True, 'shape': 'oval'})
            apply(Manage_Rotatable_Bonds_23.inputPortByName['check_for_fragments'].configure, (), {'color': 'yellow', 'cast': True, 'shape': 'circle'})
            apply(Manage_Rotatable_Bonds_23.inputPortByName['bonds_to_inactivate'].configure, (), {'color': 'white', 'cast': True, 'shape': 'oval'})
            apply(Manage_Rotatable_Bonds_23.inputPortByName['limit_torsions'].configure, (), {'color': 'white', 'cast': True, 'shape': 'oval'})
            apply(Manage_Rotatable_Bonds_23.outputPortByName['mols'].configure, (), {'color': '#c64e70', 'shape': 'oval'})
        except:
            print "WARNING: failed to restore ManageRotatableBonds named Manage Rotatable Bonds in network self.macroNetwork"
            print_exc()
            Manage_Rotatable_Bonds_23=None

        try:
            ## saving node Prepare AD4Molecule ##
            from AutoDockTools.VisionInterface.PrepareAD4Molecule import PrepareAD4Molecule
            Prepare_AD4Molecule_24 = PrepareAD4Molecule(constrkw = {}, name='Prepare AD4Molecule', library=adtlib)
            self.macroNetwork.addNode(Prepare_AD4Molecule_24,217,90)

            ## saving connections for network Prepare AD4Molecule ##
            Prepare_AD4Molecule_24.macroNetwork.freeze()
            Prepare_AD4Molecule_24.macroNetwork.unfreeze()
            Prepare_AD4Molecule_24.shrink()
        except:
            print "WARNING: failed to restore PrepareAD4Molecule named Prepare AD4Molecule in network self.macroNetwork"
            print_exc()
            Prepare_AD4Molecule_24=None

        self.macroNetwork.freeze()

        ## saving connections for network Prepare AD4 Ligands ##
        output_Ports_13 = self.macroNetwork.opNode
        if Manage_Rotatable_Bonds_23 is not None and output_Ports_13 is not None:
            try:
                self.macroNetwork.connectNodes(
                    Manage_Rotatable_Bonds_23, output_Ports_13, "mols", "new", blocking=True)
            except:
                print "WARNING: failed to restore connection between Manage_Rotatable_Bonds_23 and output_Ports_13 in network self.macroNetwork"
        input_Ports_12 = self.macroNetwork.ipNode
        if input_Ports_12 is not None and Prepare_AD4Molecule_24 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_12, Prepare_AD4Molecule_24, "new", "Remove Water Residues_molecules", blocking=True)
            except:
                print "WARNING: failed to restore connection between input_Ports_12 and Prepare_AD4Molecule_24 in network self.macroNetwork"
        if Prepare_AD4Molecule_24 is not None and Manage_Rotatable_Bonds_23 is not None:
            try:
                self.macroNetwork.connectNodes(
                    Prepare_AD4Molecule_24, Manage_Rotatable_Bonds_23, "AD4_typer_typed_mols", "mols", blocking=True)
            except:
                print "WARNING: failed to restore connection between Prepare_AD4Molecule_24 and Manage_Rotatable_Bonds_23 in network self.macroNetwork"
        self.macroNetwork.unfreeze()

        Prepare_AD4_Ligands_11.shrink()

        ## reset modifications ##
        Prepare_AD4_Ligands_11.resetTags()
        Prepare_AD4_Ligands_11.buildOriginalList()



#############################################################################
#
# Author: Ruth HUEY, Michel F. SANNER
#
# Copyright: M. Sanner TSRI 2002
#
#############################################################################


# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/TrajParser.py,v 1.2 2003/08/29 17:55:18 sophiec Exp $
#
# $Id: TrajParser.py,v 1.2 2003/08/29 17:55:18 sophiec Exp $
#
#
#
#
#
#
#

"""
This Object parses the result of a trajectory AutoDock run and returns a dictionary. 

"""
import os
from string import find, join, replace, split, rfind
import re
from MolKit.pdbParser import PdbqParser

from AutoDockTools.ResultParser import ResultParser


class TrajParser(ResultParser):
    """ reads log from a AutoDock traj docking and return structured data"""

    keywords = ResultParser.keywords + [
        'run',
        'cycle',
        'temp',
        'acc_rej_code',
        #'intermol_energy',  #(1) 
        #'internal_energy',  #(2) NB: 1+2->final docked energy
        #'e_total',          #1+2 THIS IS THE SAME AS docking_energy????

        ]
        

    def __init__(self, dlgFile=None):
        """selected dlgFile,ok sets which docked conformations to show"""
        ResultParser.__init__(self)
        self.filename = dlgFile
        #set up dict here
        if dlgFile:
            self.filename = os.path.basename(dlgFile)
            self.parse(dlgFile)


    def parse(self, filename):
        """
        to parse uses keys:
            'ntorsions',
            'run',
            'cycle',
            'temp',
            'state'
        after parsing: 
            self.clist is list of dictionaries for states,
        """
        self.filename = filename
        #reset
        self.clist = []
        dlgptr = open(filename, 'r')
        allLines =  dlgptr.readlines()
        #first 4 lines are ntorsions, run, cycle and temp
        for i in range(len(allLines)):
            l = allLines[i]
            ll = split(l)
            if 'ntorsions'==ll[0]:
                self.num_torsions = int(ll[1])
            elif 'run' == ll[0]:
                self.run = int(ll[1])
            elif 'cycle' == ll[0]:
                self.cycle = int(ll[1])
            elif 'temp' == ll[0]:
                self.temp = float(ll[1])
            elif 'state' == ll[0]:
                self.getTrajState(allLines[i:])


    def getTrajState(self, lines):
        # BUILD A DICTIONARY and put it in clist
        d = {}
        d['num_torsions'] = self.num_torsions
        #lines[i] has format:
        #state 1 A e_total x y z qx qy qz qw
        #torsion 1
        # etc
        #WHEREAS in 'test-1.dlg' State= + 17 items: 3 trans, 4quat + ndihe(10) torsions
        xx = split(lines[0])

        # remove possible punctuation
        for ind in range(len(xx)):
            if xx[ind][-1]==',': xx[ind] = xx[ind][:-1]
            if xx[ind][-1]=='.': xx[ind] = xx[ind][:-1]

        d['cycle'] = int(xx[1])
        d['acc_rej_code'] = xx[2]
        d['docking_energy'] = float(xx[3])
        d['trn_x'] = float(xx[-7])
        d['trn_y'] = float(xx[-6])
        d['trn_z'] = float(xx[-5])
        d['qtn_nx'] = float(xx[-4])
        d['qtn_ny'] = float(xx[-3])
        d['qtn_nz'] = float(xx[-2])
        d['qtn_ang_deg'] = float(xx[-1])

        angList = []
        #NB: here torsions are NOT in the same line
        for i in range(1, self.num_torsions+1):
            angList.append(float(split(lines[i])[0]))
        d['torsion_values'] = angList
        self.clist.append(d)
        return 




#!/usr/bin/env python
#
# 
##############################################################################
#
# Authors: Stefano FORLI , Ruth HUEY
#
# Copyright: A. Olson TSRI 2011
#
#############################################################################

# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/process_VSResultsFox.py,v 1.1 2011/05/05 18:26:21 rhuey Exp $
#
# $Id: process_VSResultsFox.py,v 1.1 2011/05/05 18:26:21 rhuey Exp $ 
#

"""
This script creates a pdbqt+ file containing docked coordinates, docked
energy as well as  other important summary information about the docking. 
Input for the script is a 'directory' containing 1 or more docking log files
(*.dlg) from AutoDock calculations and the 'receptor filename'. The script
reclusters the results and includes a summary of the clustering in the pdbqt+.
"""
import os, glob, time
from AutoDockTools.Docking import FoxResultProcessor


if __name__ == '__main__':
    import sys
    import getopt

    #'d:t:f:r:BLx:Dnc vh'

    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: process_VSResultsFox.py -d directory"
        print
        print "    Description of command..."
        print "         -r     receptor filename "
        print "         -d     directory"
        print "    Optional parameters:"
        print "        [-t]    rmsd tolerance (default is 2.0)"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'r:d:t:vh')
    except getopt.GetoptError, msg:
        print 'process_VSResultsFox.py: %s' %msg
        usage()
        sys.exit(2)
    
    # initialize required parameters
    #-d: directory
    directory =  None
    #-r receptor_filename 
    receptor_filename = None

    # optional parameters
    #-t: rms_tolerance
    rms_tolerance =  2.0

    # -v verbose output
    verbose = None

    # r:d:t:vh
    for o, a in opt_list:
        if o in ('-r', '--r'):
            receptor_filename = a
            if verbose: print 'set receptor_filename to ', receptor_filename
        if o in ('-d', '--d'):
            directory = a
            if verbose: print 'set directory to ', a
        if o in ('-t', '--t'):
            rms_tolerance = float(a)
            if verbose: print 'set rms_tolerance to ', rms_tolerance
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    if not directory:
        print 'process_VSResultsFox: directory must be specified.'
        usage()
        sys.exit()

    if receptor_filename is None:
        print 'process_VSResultsFox: receptor_filename must be specified.'
        usage()
        sys.exit()
        
    frp = FoxResultProcessor(receptor_filename)
    frp.process(directory=directory, rms_tolerance=rms_tolerance) 




#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/write_models_from_states.py,v 1.4 2010/04/05 21:07:42 rhuey Exp $
#
# $Id: write_models_from_states.py,v 1.4 2010/04/05 21:07:42 rhuey Exp $
#
import os, sys, sys
from string import join
from MolKit import Read
from mglutil.math.statetocoords import StateToCoords
from AutoDockTools.Conformation import Conformation

from AutoDockTools.Docking import Docking


if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: write_modes_from_states.py -l ligandfile -s statefile -o multimodelfile"
        print
        print "    Description of command..."
        print "         -l     ligandfile name"
        print "         -s     statefile name"
        print "    Optional parameters:"
        print "        [-o]    multimodel output filename "
        print "        [-S]    single string replacing statefile eg:"
        print "    'State: 29.303 14.415 23.603 0.5609 0.4518 0.2662 -0.6406 -20.89 -0.65 81.86 -17.36 28.83 -10.80 -23.98 114.21'"
        print "        [-e]    statefile includes energy"
        print "        [-z]    use zero origin"
        print "        [-i]    interim state->apply quaternion before 'about' translation"
        print "        [-n]    index of energy on energy line: default is 8"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'l:S:s:o:n:ezivh')
    except getopt.GetoptError, msg:
        print 'write_modes_from_states.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-l: ligandfile name
    ligandfile =  None
    #-S: state
    SINGLESTATE =  None #0? or False?
    #-s: statefile name
    statefile =  None
    #-o multimodel outputfilename
    outputfile = None
    #-e states_have_energy
    states_have_energy = 0
    #-n index of energy
    index_of_energy = 8
    #-z use_zero_origin 
    use_zero_origin = False
    #-i interim_state 
    interim_state = False


    # initialize optional parameter
    #-v verbose best only
    verbose = False

    #'s:o:evh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-l', '--l'):
            ligandfile = a
            if verbose: print 'set ligandfile to ', a
        if o in ('-S', '--S'):
            SINGLESTATE = 1
            if verbose: print 'set SINGLESTATE to ', a
            #['30.691_15.206_23.914_-0.3807_0.0201_0.9215_0.0747_-7.06_93.27_127.02_-130.67_7.04_-95.44_-4.91_-126.85']"
            if args[0].find('_')>-1:
                args[0] = args[0].replace("_"," ")
            states = ["State: " + join(args)]
        if o in ('-s', '--s'):
            statefile = a
            if verbose: print 'set statefile to ', a
        if o in ('-o', '--o'):
            outputfile = a
            if verbose: print 'set outputfile to ', a
        if o in ('-e', '--e'):
            states_have_energy = True
            if verbose: print 'set states_have_energy to ', states_have_energy
        if o in ('-n', '--n'):
            index_of_energy = int(a)
            if verbose: print 'set index_of_energy to ', index_of_energy
        if o in ('-z', '--z'):
            use_zero_origin = True
            if verbose: print 'set use_zero_origin to ', use_zero_origin
        if o in ('-i', '--i'):
            interim_state = True
            if verbose: print 'set interim_state to ', interim_state
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    if not ligandfile:
        print 'write_modes_from_states.py: ligandfile must be specified.'
        usage()
        sys.exit()

    if not statefile and not SINGLESTATE:
        print 'write_modes_from_states.py: SINGLESTATE or statefile must be specified.'
        usage()
        sys.exit()

    if not outputfile:
        if verbose: print 'write_modes_from_states.py: outputfile not specified. Using stdout'
        #usage()
        #sys.exit()


    lig = Read(ligandfile)
    if not len(lig):
        print "no ligand found in ", ligandfile
        sys.exit()
    lig = lig[0]
    if not hasattr(lig, 'ndihe'):
        print ligandfile + "molecule has no torsion tree"
        sys.exit()
    lig.buildBondsByDistance()
    # add extra slot to ._coords for changing coordinates
    lig.allAtoms.addConformation(lig.allAtoms.coords)
    #?is this necessary
    lig.allAtoms.setConformation(1)
    ntors = lig.ndihe
    length_of_state = 7+lig.ndihe
    # @@ handle to the input ligLines
    ligLines = lig.parser.allLines

    #setup StateToCoords object
    origin = lig.getCenter()
    if use_zero_origin or interim_state:
        origin = [0.,0.,0.]    
    #note: index of _coords to use is always 1
    lig.stoc = StateToCoords(lig, origin, 1)
    outptr = sys.stdout
    if outputfile:
        outptr = open(outputfile, 'w')


    #if SINGLESTATE:
        # eg:
        #"State: 29.303 14.415 23.603 0.5609 0.4518 0.2662 -0.6406 -20.89 -0.65 81.86 -17.36 28.83 -10.80 -23.98 114.21"
        #states = state.split()
        #['State:', '29.303', '14.415', '23.603', '0.5609', '0.4518', '0.2662', '-0.6406', '-20.89', '-0.65', '81.86', '-17.36', '28.83', '-10.80', '-23.98', '114.21']

    if statefile:
        sptr = open(statefile)
        states = sptr.readlines()
        sptr.close()
        if not len(states):
            print "no states found in ", statefile
            sys.exit()

    state_list = []
    ctr = 1
    count = len(states)
    if not SINGLESTATE and states_have_energy:
        for i in range(0,len(states),2):
            sline = states[i]
            eline =states[i+1]
            #build a states from each of the lines in statefile
            #State:\t  4.847  -2.386  14.760  -0.413  0.552 -0.724  4.257     58.27  -33.47  -87.92  134.64  -36.46  114.79  -44.86  -74.96 -118.53   77.29  139.08   78.23  -52.09  -12.69   35.08 -118.21 -175.94\n'
            fl = map(float, sline.split()[1:])
            assert len(fl)==length_of_state
            # 0  1  2  3  4  5  6 [7....
            #[t1,t2,t3,q1,q2,q3,q4,tors1,tors2, tors3....
            #
            translation = fl[:3]
            quaternion = [fl[6],fl[3:6]]
            #energy = eline.split()[8]
            energy = eline.split()[index_of_energy]
            torsion_angles = []
            if ntors>0:
                torsion_angles = fl[7:]
                newConf = Conformation(lig,origin,translation, quaternion, torsion_angles)
                newCrds = newConf.getCoords()
                if interim_state:
                    #here's where to add back the origin or ?
                    newCrds -= origin
                #write some MODEL stuff then newCrds
                ostr = "MODEL %d\n"%ctr
                outptr.write(ostr)
                ctr += 1
                ostr = "REMARK AD4 RESULT: %s\n" %energy #put energy here...
                outptr.write(ostr)
                #lig.parser.write_with_new_coords(newCrds,outptr)
                ct = 0
                for l in ligLines:
                    if l.find("ATOM")==0 or l.find("HETATM")==0:
                        cc = newCrds[ct]
                        ct = ct + 1
                        new_l = l[:30]+"%8.3f%8.3f%8.3f" %(cc[0],cc[1],cc[2]) + l[54:]
                    else:
                        new_l = l
                    outptr.write(new_l) 
                if verbose: print "wrote ", outputfile
                ostr = "ENDMDL\n"
                outptr.write(ostr)
                i+=1
    else:
        for sline in states:
            #build a state from each of the lines in states
            #State:\t  4.847  -2.386  14.760  -0.413  0.552 -0.724  4.257     58.27  -33.47  -87.92  134.64  -36.46  114.79  -44.86  -74.96 -118.53   77.29  139.08   78.23  -52.09  -12.69   35.08 -118.21 -175.94\n'
            fl = map(float, sline.split()[1:])
            assert len(fl)==length_of_state
            # 0  1  2  3  4  5  6 [7....
            #[t1,t2,t3,q1,q2,q3,q4,tors1,tors2, tors3....
            #
            translation = fl[:3]
            # interpret axis_angle as quaternion x y z w
            # use quaternion as w, (x,y,z) for mglutil/math/transformation.py class
            quaternion = [fl[6],fl[3:6]]
            torsion_angles = []
            if ntors>0:
                torsion_angles = fl[7:]
                newConf = Conformation(lig,origin,translation, quaternion, torsion_angles)
                newCrds = newConf.getCoords()
                #write some MODEL stuff then newCrds
                ostr = "MODEL %d\n"%ctr
                outptr.write(ostr)
                ctr += 1
                ostr = "REMARK AD4 RESULT: n/a\n" #put energy here...
                outptr.write(ostr)
                #lig.parser.write_with_new_coords(newCrds,outptr)
                ct = 0
                for l in ligLines:
                    if l.find("ATOM")==0 or l.find("HETATM")==0:
                        cc = newCrds[ct]
                        ct = ct + 1
                        new_l = l[:30]+"%8.3f%8.3f%8.3f" %(cc[0],cc[1],cc[2]) + l[54:]
                    else:
                        new_l = l
                    outptr.write(new_l) 
                if verbose: print "wrote ", outputfile
                ostr = "ENDMDL\n"
                outptr.write(ostr)
    if verbose: print "Done!"
    outptr.close()            

# To execute this command type:
# write_modes_from_states.py -d docking_filename 
# optional arguments
# -o outputfile_stem (default is ligandname)



## Automatically adapted for numpy.oldnumeric Jul 23, 2007 by 

#
# Last modified on Thu Aug  9 16:32:21 PDT 2001 by lindy
#
# This is the python translation of cluster_analysis.cc from
# the AutoDock code.

#import numpy.oldnumeric as N
import numpy.oldnumeric as Numeric
N = Numeric
import UserList
from mglutil.math.rmsd import RMSDCalculator
from Pmv.setangleCommands import SetRelativeTorsion, SetQuaternion

class Clust(UserList.UserList):
    def __init__(self, mol, seed, coords):
        UserList.UserList.__init__(self)
        self.seed = seed
        self.mol = mol
        self.seed_coords = coords 
        self.ruler = RMSDCalculator(coords)
        self.min_energy = self.max_energy = mol.autodock_states[seed].e_binding ##### get seed energy HERE
        self.append(seed)

    def append(self, item):
        # override append to maintain min, max
        if self.mol.autodock_states[item].e_binding < self.min_energy:
            self.min_energy = self.mol.autodock_states[item].e_binding
        if self.mol.autodock_states[item].e_binding > self.max_energy:
            self.max_energy = self.mol.autodock_states[item].e_binding
        self.data.append(item)
   
class Cluster_AD:
    """
    """
    def __init__(self, mol):
        self.mol = mol
        energies = []
        for state in mol.autodock_states:
            energies.append(state.e_binding)
        self.energies = N.array(energies)
        self.argsort = N.argsort(self.energies)
        self.test_coords = None
        self.rmsdarray=N.zeros([len(self.mol.autodock_states),
                                len(self.mol.autodock_states)])-1.
        
    def set_reference(self, reference=None):
        """reference should be an index into the data array.
        If it's a problem to have the reference be part of the data
        array, reference could be a separate instance of data.
        """
        if not reference:
            self.reference = self.argsort[0]  # reference lowest energy conformation
        else:
            self.reference = reference
        self.get_coords(self.mol, self.reference)
       
    def set_tolerance(self, tolerance):
        self.tolerance = tolerance

    def get_dist(self, clust, probeidx):
        if self.rmsdarray[clust.seed, probeidx] == -1:  
            self.get_coords(self.mol, probeidx) # sets self.test_coords
            clust.ruler.computeRMSD(self.test_coords)
            self.rmsdarray[clust.seed, probeidx] = clust.ruler.rmsd
        return self.rmsdarray[clust.seed, probeidx]

    def get_coords(self, mol, val):
        SetTors = SetRelativeTorsion()
        SetQuat = SetQuaternion()
        mol.torTree.fillTorAtoms(mol)
        #BEFORE CALCULATING NEW COORDS: RESET TO initial ones
        if len(mol.allAtoms[0]._coords)==1:
            mol.allAtoms.addConformation(mol.allAtoms.coords)
            mol.allAtoms.setConformation(1)
        else:
            for i in range(len(mol.allAtoms)):
                at = mol.allAtoms[i]
                at._coords[at.conformation] = at._coords[0]
        #do something special for 0
        if val!=-1:
            state = mol.autodock_states[val]
            tList = mol.torTree.tList
            angList = state.torsions
            allAtoms = mol.allAtoms
            for k in range(state.ntorsions):
                thisT = tList[k]
                at1 = allAtoms[thisT.atm1_ix]
                at2 = allAtoms[thisT.atm2_ix]
                SetTors.doit(at1, at2, angList[k], 
                             thisT.mov_atoms)
            SetQuat.doit(allAtoms, state.quaternion, 
                         state.origin, state.translation)
        self.test_coords = mol.allAtoms.coords

    def make_clusters(self, tolerance=None, ref=None):
        if tolerance:
            self.set_tolerance(tolerance)
        self.set_reference(ref) # set self.test_coords
        clusters = []
        clusters.append(Clust(self.mol, self.reference, self.test_coords))
        self.test_coords = None # reset self.test_coords
        # Go through all conformations
        for i in range(len(self.argsort)):
            probe_conf_ix = self.argsort[i]
            if probe_conf_ix == self.reference: continue
            #Assume this is a new conformation until proven otherwise...
            new_conf = 1
            for c in clusters:
                #get dist to lowest energy in cluster (seed)
                dist = self.get_dist(c, probe_conf_ix)
                #Check rms; if greater than tolerance
                if dist > self.tolerance:
                    continue
                else: # add this conformation to the current cluster
                    c.append(probe_conf_ix)
                    new_conf = 0
                    break
            if new_conf: # start a new cluster
                if not self.test_coords:
                    self.get_coords(self.mol, probe_conf_ix)
                clusters.append(Clust(self.mol, probe_conf_ix, self.test_coords))
            self.test_coords = None
        self.clusters = clusters
        return self.clusters
















#############################################################################
#
# Author: J. Ren
#
#############################################################################

"""
Receptor - can be a pdb, pqr, or pdbqt
"""

import os

class receptor:

    def __init__(self, filename=None):
        self.ext_loc = {'pdb': [None, None], 'pqr': [None, None], 'pdbqt': [None, None]}
        self.workdir = None
        self.id = None

        if filename != None:
            self.set_ext_loc(filename)
            self.set_workdir(filename)
 
    def set_workdir(self, d):
        if not(d.startswith('http://')):
            d = os.path.abspath(d)

            if d.endswith('.pdb') or d.endswith('.pqr') or d.endswith('.pdbqt'):
                d = os.path.dirname(d)

            self.workdir = d

    def set_id(self, id):
        bn = os.path.basename(id)
        self.id = bn.split(os.extsep)[0]

    def set_ext_loc(self, filename):
        ext = os.path.splitext(filename)[1].strip(os.extsep)
        self.set_id(filename)        

        if filename.startswith('http://'):
            self.ext_loc[ext] = [filename, 'url']
        else:
            full_path = os.path.abspath(filename)
            dir_name = os.path.dirname(full_path)
            
            ext_list = ['pdb', 'pqr', 'pdbqt']

            for i in ext_list:
                f = dir_name + os.sep + self.id + os.extsep + i
                if os.path.exists(f):
                    self.ext_loc[i] = [f, 'local']
                    self.set_workdir(full_path)

    def get_workdir(self):
        if self.workdir == None:
            exts = ['pdb', 'pqr', 'pdbqt']
            
            for i in exts:
                if self.get_ext_type(i) == "local":
                    self.set_workdir(self.get_ext_loc(i))
                    break

        return self.workdir

    def get_id(self):
        return self.id

    def get_ext_loc(self, ext):
        return self.ext_loc[ext][0]

    def get_ext_type(self, ext):
        return self.ext_loc[ext][1] 




#########################################################################
#
# Date: Nov 2001 Authors: Michel Sanner
#
#    sanner@scripps.edu
#
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Michel Sanner and TSRI
#
#########################################################################

from NetworkEditor.items import NetworkNode

from AutoDockTools.VisionInterface.Adt.LigandDB import LigandDB
from mglutil.util.packageFilePath import getResourceFolderWithVersion

import os
import time
import urllib2

class PublicServerLigandDB(NetworkNode):
    """
    List of available public libraries on the virtual screening server.
    A description of the ligand libraries can be found on
    http://nbcr.sdsc.edu/pub/wiki/index.php?title=Virtual_Screening_Libraries

    Input: a public ligand library name
    Output: LigandDB object containing info about the info
    """
    
    def __init__(self, name='PublicServerLigandDB', **kw):
        import urllib

        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )

        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )
        ip = self.inputPortsDescr
        ip.append(datatype='string', name='server_lib', required=True, )

        fqdn = "kryptonite.nbcr.net"
        url = "http://" + fqdn + "/pub_ligand_libs.txt"

        publibdir = os.path.join(getResourceFolderWithVersion(), 'ws')

        if not (os.path.exists(publibdir)):
            os.mkdir(publibdir)
        
        publiblocal = os.path.join(publibdir, 'publibs.txt')

        lock = publiblocal + '.lock'

        if os.path.exists(lock) and time.time() - os.path.getmtime(lock) > 15:
            os.remove(lock)
            
        try:
            if not(os.path.exists(lock)):
                open(lock, 'w').close()
                publibweb = urllib2.urlopen(url)
                outfile = open(publiblocal, 'w')
                outfile.write(publibweb.read())
                outfile.close()
                os.remove(lock)
        except:
            print "[INFO]: Getting list of public server libs from cache"
            pass
            

        try:
            f = open(publiblocal, 'r')
            self.choices = f.read().split()
            f.close()
        except:
            self.choices = []
            print "[ERROR]: Unable to public server libs from the web and from cache"

        self.widgetDescr['server_lib'] = {
            'class':'NEComboBox', 'master':'node',
            'choices':self.choices,
            'fixedChoices':True,
            'entryfield_entry_width':18,
            'labelGridCfg':{'sticky':'w'},
            'widgetGridCfg':{'sticky':'w'},
            'labelCfg':{'text':'Server Libraries:'}}

        op = self.outputPortsDescr
        op.append(datatype='LigandDB', name='ligDB')

        code = """def doit(self, server_lib):
    ligDB = LigandDB(server_lib=server_lib)

    self.outputData(ligDB=ligDB)
"""
        self.setFunction(code)



## Automatically adapted for numpy.oldnumeric Jul 23, 2007 by 

#############################################################################
#
# Author: Ruth HUEY, Michel F. SANNER
#
# Copyright: M. Sanner TSRI 2005
#
#############################################################################

#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/pyAutoDockCommands.py,v 1.21 2008/07/15 22:24:11 rhuey Exp $
#
# $Id: pyAutoDockCommands.py,v 1.21 2008/07/15 22:24:11 rhuey Exp $
#
#
#
#
#
#
#
"""
This Module contains commands which allow the user to compute various
component pair-wise energies of the autodock305 forcefield
and assign the results to individual atoms. 

"""

import Tkinter, numpy.oldnumeric as Numeric, Pmw

from PyAutoDock.MolecularSystem import MolecularSystem
from PyAutoDock.scorer import WeightedMultiTerm, Distance
from PyAutoDock.vanDerWaals import VanDerWaals 
from PyAutoDock.vanDerWaals import HydrogenBonding, HydrogenBonding12_10
from PyAutoDock.vanDerWaals import NewHydrogenBonding, NewHydrogenBonding12_10
from PyAutoDock.InternalEnergy import InternalEnergy
from PyAutoDock.electrostatics import Electrostatics
from PyAutoDock.desolvation import Desolvation, NewDesolvation
from PyAutoDock.AutoDockScorer import AutoDock305Scorer, AutoDock4Scorer
from PyAutoDock.AutoDockScorer import AutoDockTermWeights305, AutoDockTermWeights4

from Pmv.mvCommand import MVCommand, MVAtomICOM, MVBondICOM
from Pmv.stringSelectorGUI import StringSelectorGUI
from MolKit import Read
from MolKit.molecule import AtomSet, Atom, BondSet
from ViewerFramework.VFCommand import Command, CommandGUI
from DejaVu.Geom import Geom
from mglutil.gui.InputForm.Tk.gui import InputFormDescr
from mglutil.gui.BasicWidgets.Tk.thumbwheel import ThumbWheel
from mglutil.gui.BasicWidgets.Tk.customizedWidgets import ExtendedSliderWidget
from mglutil.util.misc import ensureFontCase


parentGeom = Geom("pyADGeoms", shape=(0,0), pickable=0)
parentGeom.isScalable = 0
parentGeom.in_viewer = False

pep_aromList  = ['PHE_CD1', 'PHE_CG', 'PHE_CD2', 'PHE_CE1',\
             'PHE_CE2', 'PHE_CZ', 'TYR_CD1', 'TYR_CG', 'TYR_CD2', 'TYR_CE1',\
             'TYR_CE2', 'TYR_CZ', 'HIS_CD2', 'HIS_CE1', 'HIS_CG', 'TRP_CD1',\
             'TRP_CG', 'TRP_CD2', 'TRP_CE2', 'TRP_CZ2', 'TRP_CH2', 'TRP_CZ3',\
             'TRP_AE3', 'PHE_AD1', 'PHE_AG', 'PHE_AD2', 'PHE_AE1',\
             'PHE_AE2', 'PHE_AZ', 'TYR_AD1', 'TYR_AG', 'TYR_AD2', 'TYR_AE1',\
             'TYR_AE2', 'TYR_AZ', 'HIS_AD2', 'HIS_AE1', 'HIS_AG', 'TRP_AD1',\
             'TRP_AG', 'TRP_AD2', 'TRP_AE2', 'TRP_AZ2', 'TRP_AH2', 'TRP_AZ3',\
             'TRP_AE3']


class PyADCalcClosestPairs(MVCommand):
    """For each atom in one AtomSet, determine the index of closest atom in a second
AtomSet and the distance between them.  A display option exists which draws
lines between each atom in one set and the closest atom to it in the second
set which are closer than 'cutoff' angstrom distance.  By default, cutoff is
set to 8 angstrom but can be set by the user.  The color of the lines drawn
from the atoms in first set to their closest atom in the second set can be
customized. Similarly, the color of the lines drawn from the atoms in the
second set to their closest atom in the first set can be customized."""

    def onAddCmdToViewer(self):
        from DejaVu.IndexedPolylines import IndexedPolylines
        miscGeom = self.vf.GUI.miscGeom
        # we don't need that that check anymore,
        # as miscGeom is always added when we instanciate ViewerFrameworkGUI
        #if miscGeom not in self.vf.GUI.VIEWER.rootObject.children:
        #    self.vf.GUI.VIEWER.AddObject(miscGeom)
        
        
        if not parentGeom.in_viewer:
            self.vf.GUI.VIEWER.AddObject(parentGeom, parent=miscGeom)
            parentGeom.in_viewer = True
        self.first_lines = IndexedPolylines('first_CloseAts', 
                    materials = ((0,1,1),), lineWidth=4, 
                    stippleLines=1, inheritMaterial=0)
        self.second_lines = IndexedPolylines('second_CloseAts', 
                    materials = ((0,1,0),), lineWidth=4, 
                    stippleLines=1, inheritMaterial=0)
        self.vf.GUI.VIEWER.AddObject(self.first_lines, parent=parentGeom)
        self.vf.GUI.VIEWER.AddObject(self.second_lines, parent=parentGeom)


    def onRemoveObjectFromViewer(self, obj):
        ats = obj.findType(Atom)
        ats_with_closest_at = ats.get(lambda x: hasattr(x, 'closest_at'))
        #remove the atoms linked to these atoms
        if len(ats_with_closest_at):
            delattr(ats_with_closest_at, 'closest_at')
            self.first_lines.Set(visible=False)
            self.second_lines.Set(visible=False)
        #remove these atoms from any linked to them
        ats_with_closest_at = self.vf.allAtoms.get(lambda x: hasattr(x, 'closest_at'))
        #just remove all of them for simplicity's sake... instead of looping
        #over them checking whether to remove each one...
        if len(ats_with_closest_at):
            delattr(ats_with_closest_at, 'closest_at')

            
    def buildForm(self):
        ifd = self.ifd = InputFormDescr(title = "For Closest Pairs: Specify 'first' and 'second' group of atoms:")
        self.hideDSel = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        self.hideASel = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        self.displayClosePairsVar = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        self.useSelection = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        ifd.append({'name':'DkeyLab',
                    'widgetType':Tkinter.Label,
                    'text':'For First atoms:',
                    'gridcfg':{'sticky':'w','columnspan':3}})
        ifd.append({'name':'selDRB0',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Use all atoms',
                'variable': self.hideDSel,
                'value':1,
                'command':self.hideDSelector
                },
            'gridcfg':{'sticky':'w'}})
        ifd.append({'name':'selDRB1',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Set atoms to use',
                'variable': self.hideDSel,
                'value':0,
                'command':self.hideDSelector
                },
            'gridcfg':{'sticky':'w', 'row':-1, 'column':1}})
        ifd.append({'name': 'keyDSelector',
            'wtype':StringSelectorGUI,
            'widgetType':StringSelectorGUI,
            'wcfg':{ 'molSet': self.vf.Mols,
                    'vf': self.vf,
                    'all':1,
                    'crColor':(0.,1.,.2),
            },
            'gridcfg':{'sticky':'we', 'columnspan':3 }})
        #now the acceptors
        ifd.append({'name':'AkeyLab',
                    'widgetType':Tkinter.Label,
                    'text':'For Second atoms:',
                    'gridcfg':{'sticky':'w','columnspan':3}})
        ifd.append({'name':'selARB0',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Use all atoms',
                'variable': self.hideASel,
                'value':1,
                'command':self.hideASelector
                },
            'gridcfg':{'sticky':'w'}})
        ifd.append({'name':'selARB1',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Set atoms to use',
                'variable': self.hideASel,
                'value':0,
                'command':self.hideASelector
                },
            'gridcfg':{'sticky':'w', 'row':-1, 'column':1}})
        ifd.append({'name': 'keyASelector',
            'wtype':StringSelectorGUI,
            'widgetType':StringSelectorGUI,
            'wcfg':{ 'molSet': self.vf.Mols,
                    'vf': self.vf,
                    'all':1,
                    'crColor':(0.,1.,.2),
            },
            'gridcfg':{'sticky':'we', 'columnspan':3 }})
        ifd.append({'name': 'showClosePairs',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text': 'Display Close Pairs',
                    'variable': self.displayClosePairsVar,
                    'value':1,
                    'command':self.hideDisplay,
                    },
            'gridcfg':{'sticky':Tkinter.W}})
        ifd.append({'name': 'hideClosePairs',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text': 'Hide Close Pairs',
                    'variable': self.displayClosePairsVar,
                    'value':0,
                    'command':self.hideDisplay,
                    },
            'gridcfg':{'sticky':'w', 'row':-1, 'column':1}})
        ifd.append( {'name':'cutoff',
            'widgetType': ExtendedSliderWidget,
            'wcfg':{'label':'cutoff',
                'minval':1.0, 'maxval':20.0,
                'immediate':0,
                'init':8.,
                'labelsCursorFormat':'%3.1f',
                'entrywcfg':{'width':4},
                'width':250,
                'sliderType':'float',
                'entrypackcfg':{'side':'right'}},
            'gridcfg':{'sticky':'wens', 'columnspan':3}})
        ifd.append({'name':'colorLab',
                    'widgetType':Tkinter.Label,
                    'text':'Set color of lines from:',
                    'gridcfg':{'sticky':'w','columnspan':3}})
        ifd.append({'name':'firstColorLab',
                    'widgetType':Tkinter.Label,
                    'text':'First atoms to closest atom in Second set:',
                    'gridcfg':{'sticky':'w','columnspan':3}})
        ifd.append( {'name':'rvalCtr1',
            'widgetType': Pmw.Counter,
            'wcfg':{
            'labelpos' : 'w',
            'label_text' : 'R:', 
            'entry_width' : 4,
            'entryfield_value' : '1.0',
            'datatype' :  {'counter':'real',
                        'separator': '.'},
            'entryfield_validate' : {'validator' : 'real',
                                           'min' : 0., 'max' : 1.0,
                                           'separator': '.'},
            'increment':0.1},
            'gridcfg':{'sticky':'w',}})
        ifd.append( {'name':'gvalCtr1',
            'widgetType': Pmw.Counter,
            'wcfg':{
            'labelpos' : 'w',
            'label_text' : 'G:', 
            'entry_width' : 4,
            'entryfield_value' : '1.0',
            'datatype' :  {'counter':'real',
                        'separator': '.'},
            'entryfield_validate' : {'validator' : 'real',
                                           'min' : 0., 'max' : 1.0,
                                           'separator': '.'},
            'increment':0.1},
            'gridcfg':{'sticky':'w','row':-1, 'column':1}})
        ifd.append( {'name':'bvalCtr1',
            'widgetType': Pmw.Counter,
            'wcfg':{
            'labelpos' : 'w',
            'label_text' : 'B:', 
            'entry_width' : 4,
            'entryfield_value' : '1.0',
            'datatype' :  {'counter':'real',
                        'separator': '.'},
            'entryfield_validate' : {'validator' : 'real',
                                           'min' : 0., 'max' : 1.0,
                                           'separator': '.'},
            'increment':0.1},
            'gridcfg':{'sticky':'w','row':-1, 'column':2}})
        ifd.append({'name':'secondColorLab',
                    'widgetType':Tkinter.Label,
                    'text':'Second atoms to closest atom in First set:',
                    'gridcfg':{'sticky':'w','columnspan':3}})
        ifd.append( {'name':'rvalCtr2',
            'widgetType': Pmw.Counter,
            'wcfg':{
            'labelpos' : 'w',
            'label_text' : 'R:', 
            'entry_width' : 4,
            'entryfield_value' : '1.0',
            'datatype' :  {'counter':'real',
                        'separator': '.'},
            'entryfield_validate' : {'validator' : 'real',
                                           'min' : 0., 'max' : 1.0,
                                           'separator': '.'},
            'increment':0.1},
            'gridcfg':{'sticky':'w',}})
        ifd.append( {'name':'gvalCtr2',
            'widgetType': Pmw.Counter,
            'wcfg':{
            'labelpos' : 'w',
            'label_text' : 'G:', 
            'entry_width' : 4,
            'entryfield_value' : '1.0',
            'datatype' :  {'counter':'real',
                        'separator': '.'},
            'entryfield_validate' : {'validator' : 'real',
                                           'min' : 0., 'max' : 1.0,
                                           'separator': '.'},
            'increment':0.1},
            'gridcfg':{'sticky':'w','row':-1, 'column':1}})
        ifd.append( {'name':'bvalCtr2',
            'widgetType': Pmw.Counter,
            'wcfg':{
            'labelpos' : 'w',
            'label_text' : 'B:', 
            'entry_width' : 4,
            'entryfield_value' : '1.0',
            'datatype' :  {'counter':'real',
                        'separator': '.'},
            'entryfield_validate' : {'validator' : 'real',
                                           'min' : 0., 'max' : 1.0,
                                           'separator': '.'},
            'increment':0.1},
            'gridcfg':{'sticky':'w','row':-1, 'column':2}})
        ifd.append({'widgetType': Tkinter.Button,
            'text':'Ok',
            'wcfg':{'bd':6},
            'gridcfg':{'sticky':'ew', 'columnspan':2 },
            'command':self.Accept_cb})
        ifd.append({'widgetType': Tkinter.Button,
            'text':'Cancel',
            'wcfg':{'bd':6},
            'gridcfg':{'sticky':'ew', 'column':2,'row':-1},
            'command':self.Close_cb})
        self.form = self.vf.getUserInput(self.ifd, modal=0, blocking=0)
        self.hideDSelector()
        self.hideASelector()
        self.hideDisplay()


    def guiCallback(self):
        if not len(self.vf.Mols):
            self.warningMsg('no molecules in viewer')
            return 
        if not hasattr(self, 'ifd'):
            self.buildForm()
        else:
            self.form.deiconify()


    def Close_cb(self, event=None):
        self.form.withdraw()


    def Accept_cb(self, event=None):
        self.form.withdraw()
        firstNodesToCheck = self.ifd.entryByName['keyDSelector']['widget'].get()
        if not len(firstNodesToCheck):
            self.warningMsg('ERROR! no atoms in first group specified')
            return "ERROR"
        else:
            firstAts = firstNodesToCheck.findType(Atom)
            if self.useSelection.get():
                curSel = self.vf.getSelection()
                if len(curSel):
                    curAts = curSel.findType(Atom)
                    if curAts != self.vf.allAtoms:
                        firstAts = curAts.inter(firstAts)
                        if firstAts is None:
                            self.warningMsg('ERROR! no atoms in first group specified')
                            return "ERROR"
                else:
                    self.warningMsg('ERROR! no atoms in first group specified')
                    return "ERROR"
        secondNodesToCheck = self.ifd.entryByName['keyASelector']['widget'].get()
        if not len(secondNodesToCheck): 
            self.warningMsg('ERROR! no atoms in second group specified')
            return "ERROR"
        else:
            secondAts = secondNodesToCheck.findType(Atom)
            if self.useSelection.get():
                curSel = self.vf.getSelection()
                if len(curSel):
                    curAts = curSel.findType(Atom)
                    if curAts != self.vf.allAtoms:
                        secondAts = curAts.inter(secondAts)
                        if not secondAts:  #NB inter returns empty set!
                            self.warningMsg('ERROR! no atoms in second group specified')
                            return "ERROR"
                else:
                    self.warningMsg('ERROR! no atoms in second group specified')
                    return "ERROR"
        self.Close_cb()
        display = self.displayClosePairsVar.get()
        cutoff = self.ifd.entryByName['cutoff']['widget'].get()
        mat1 = (float(self.ifd.entryByName['rvalCtr1']['widget'].get()),
                float(self.ifd.entryByName['gvalCtr1']['widget'].get()),
                float(self.ifd.entryByName['bvalCtr1']['widget'].get()),)
        mat2 = (float(self.ifd.entryByName['rvalCtr2']['widget'].get()),
                float(self.ifd.entryByName['gvalCtr2']['widget'].get()),
                float(self.ifd.entryByName['bvalCtr2']['widget'].get()),)
        return self.doitWrapper(firstAts, secondAts, display, cutoff, mat1, mat2)


    def hideDSelector(self, event=None):
        e = self.ifd.entryByName['keyDSelector']
        if self.hideDSel.get():
            e['widget'].grid_forget()
        else:
            e['widget'].grid(e['gridcfg'])
        self.form.autoSize() 


    def hideASelector(self, event=None):
        e = self.ifd.entryByName['keyASelector']
        if self.hideASel.get():
            e['widget'].grid_forget()
        else:
            e['widget'].grid(e['gridcfg'])
        self.form.autoSize() 


    def hideDisplay(self, event=None):
        names = ['cutoff', 'colorLab', 'firstColorLab', 'rvalCtr1', 'gvalCtr1', 'bvalCtr1', \
                    'secondColorLab','rvalCtr2', 'gvalCtr2', 'bvalCtr2'] 
        for name in names:
            e = self.ifd.entryByName[name]
            if not self.displayClosePairsVar.get():
                e['widget'].grid_forget()
                self.first_lines.Set(visible=0)
                self.second_lines.Set(visible=0)
            else:
                apply(e['widget'].grid, (), e['gridcfg'])
                self.first_lines.Set(visible=1)
                self.second_lines.Set(visible=1)
        self.form.autoSize() 
        self.vf.GUI.VIEWER.Redraw()


    def __call__(self, first, second, display=False, cutoff=8.0, \
                    mat1=(0,1,1), mat2 = (0,1,0), **kw):
        """None<-PyAD_calcClosestPairs(first, second)
first:one set of atoms
second:second set of atoms
        """
        apply(self.doitWrapper, (first, second, display, cutoff, mat1, mat2), kw)


    def doit(self, first, second, display, cutoff, mat1, mat2):
        firstAts = self.vf.expandNodes(first)
        secondAts = self.vf.expandNodes(second)
        ms = self.ms = MolecularSystem()
        ms.add_entities(firstAts) 
        ms.add_entities(secondAts) 
        dist = ms.get_dist_mat(0,1)
        #label each first atom by distance to closest second atom
        distances = map(min, dist) #this returns list of lowest vals
        #warning: min distance is always from an atom to itself...
        if display:
            vertices = []
            faces = []
            ct = 0
        for i in range(len(dist)):
            a = firstAts[i]
            min_dist = a.min_dist = distances[i]
            min_index = a.min_index = dist[i].index(min_dist)
            b = secondAts[min_index]
            a.closest_at = b
            if display:
                if min_dist < cutoff:
                    vertices.extend([a.coords, b.coords])
                    faces.append((ct, ct+1))
                    ct=ct+2
        if display:
            self.firstVerts = vertices
            self.firstFaces = faces
            self.first_lines.Set(vertices=vertices, faces=faces, materials=(mat1,))
            self.first_lines.Set(visible=True)
            vertices = []
            faces = []
            ct = 0
        #label each second atom by distance to closest first atom
        swap_dist = Numeric.swapaxes(dist, 0,1)
        column_distances = map(min, swap_dist)
        #now do the second ones
        for i in range(len(swap_dist)):
            b = secondAts[i]
            min_dist = b.min_dist = column_distances[i]
            min_index = b.min_index = swap_dist[i].tolist().index(min_dist)
            a = firstAts[min_index]
            b.closest_at = a
            if display:
                if min_dist < cutoff:
                    vertices.extend([b.coords, a.coords])
                    faces.append((ct, ct+1))
                    ct=ct+2
        if display:
            self.secondVerts = vertices
            self.secondFaces = faces
            self.second_lines.Set(vertices=vertices, faces=faces, materials=(mat2,))
            self.second_lines.Set(visible=True)
            self.vf.GUI.VIEWER.Redraw()


PyADCalcClosestPairsGUI=CommandGUI()
PyADCalcClosestPairsGUI.addMenuCommand('AutoToolsBar', 'PyAutoDock', 'calculate closest pairs')


class PyADCalcVDWEnergies(MVCommand):
    """For each atom in one AtomSet, determine the vdw energy vs all the atoms in a second
    AtomSet"""

    def __init__(self, func=None):
        MVCommand.__init__(self, func)
        self.flag = self.flag | self.objArgOnly
        self.weight = .1485
        self.scorer = VanDerWaals()
        self.prop = 'vdw_energy'
        self.title = "For VanDerWaals Energies: Specify 'first' and 'second' group of atoms:"
        self.weightLabelTxt = "VDW weight"


    def onAddCmdToViewer(self):
        self.hideDSel = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        self.hideASel = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        self.weightLabel = Tkinter.StringVar(master=self.vf.GUI.ROOT)
        self.weightLabel.set(self.weightLabelTxt)


    def buildForm(self):
        ifd = self.ifd = InputFormDescr(title = "Specify 'first' and 'second' group of atoms:")
        self.useSelection = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        ifd.append({'name':'DkeyLab',
                    'widgetType':Tkinter.Label,
                    'text':'For First atoms:',
                    'gridcfg':{'sticky':'w','columnspan':3}})
        ifd.append({'name':'selDRB0',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Use all atoms',
                'variable': self.hideDSel,
                'value':1,
                'command':self.hideDSelector
                },
            'gridcfg':{'sticky':'w'}})
        ifd.append({'name':'selDRB1',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Set atoms to use',
                'variable': self.hideDSel,
                'value':0,
                'command':self.hideDSelector
                },
            'gridcfg':{'sticky':'w', 'row':-1, 'column':1}})
        ifd.append({'name': 'keyDSelector',
            'wtype':StringSelectorGUI,
            'widgetType':StringSelectorGUI,
            'wcfg':{ 'molSet': self.vf.Mols,
                    'vf': self.vf,
                    'all':1,
                    'crColor':(0.,1.,.2),
            },
            'gridcfg':{'sticky':'we', 'columnspan':2 }})
        #now the second atoms
        ifd.append({'name':'AkeyLab',
                    'widgetType':Tkinter.Label,
                    'text':'For Second atoms:',
                    'gridcfg':{'sticky':'w','columnspan':3}})
        ifd.append({'name':'selARB0',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Use all atoms',
                'variable': self.hideASel,
                'value':1,
                'command':self.hideASelector
                },
            'gridcfg':{'sticky':'w'}})
        ifd.append({'name':'selARB1',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Set atoms to use',
                'variable': self.hideASel,
                'value':0,
                'command':self.hideASelector
                },
            'gridcfg':{'sticky':'w', 'row':-1, 'column':1}})
        ifd.append({'name': 'keyASelector',
            'wtype':StringSelectorGUI,
            'widgetType':StringSelectorGUI,
            'wcfg':{ 'molSet': self.vf.Mols,
                    'vf': self.vf,
                    'all':1,
                    'crColor':(0.,1.,.2),
            },
            'gridcfg':{'sticky':'we', 'columnspan':2 }})
        ifd.append({'name':'weight',
            'wtype':ThumbWheel,
            'widgetType':ThumbWheel,
            'wcfg':{ 'labCfg':{'text':self.weightLabel.get(), 
                        'font':(ensureFontCase('helvetica'),12,'bold')},
                'showLabel':2, 'width':100,
                'min':0.0, 'max':2, 'type':float, 'precision':4,
                'value':self.weight,
                'showLabel':1,
                'continuous':1, 'oneTurn':2, 'wheelPad':2, 'height':20},
            'gridcfg':{'sticky':'we', 'columnspan':2}})
        ifd.append({'widgetType': Tkinter.Button,
            'text':'Ok',
            'wcfg':{'bd':6},
            'gridcfg':{'sticky':'ew', 'columnspan':2 },
            'command':self.Accept_cb})
        ifd.append({'widgetType': Tkinter.Button,
            'text':'Cancel',
            'wcfg':{'bd':6},
            'gridcfg':{'sticky':'ew', 'column':2,'row':-1},
            'command':self.Close_cb})
        self.form = self.vf.getUserInput(self.ifd, modal=0, blocking=0)
        self.hideDSelector()
        self.hideASelector()


    def guiCallback(self):
        if not len(self.vf.Mols):
            self.warningMsg('no molecules in viewer')
            return 
        if not hasattr(self, 'ifd'):
            self.buildForm()
        else:
            self.form.deiconify()


    def Close_cb(self, event=None):
        self.form.withdraw()


    def Accept_cb(self, event=None):
        self.form.withdraw()
        firstNodesToCheck = self.ifd.entryByName['keyDSelector']['widget'].get()
        if not len(firstNodesToCheck):
            self.warningMsg('no first atoms specified')
            firstAts = AtomSet([])
        else:
            firstAts = firstNodesToCheck.findType(Atom)
            if self.useSelection.get():
                curSel = self.vf.getSelection()
                if len(curSel):
                    curAts = curSel.findType(Atom)
                    if curAts != self.vf.allAtoms:
                        firstAts = curAts.inter(firstAts)
                        if firstAts is None:
                            msg = 'no specified first atoms in current selection'
                            self.warningMsg(msg)
                            firstAts = AtomSet([])
                else:
                    msg = 'no current selection'
                    self.warningMsg(msg)
                    firstAts = AtomSet([])
        ###FIX THIS!!!
        secondNodesToCheck = self.ifd.entryByName['keyASelector']['widget'].get()
        if not len(secondNodesToCheck):
            self.warningMsg('no second group of atoms specified')
            secondAts = AtomSet([])
            #return
        else:
            secondAts = secondNodesToCheck.findType(Atom)
            if self.useSelection.get():
                curSel = self.vf.getSelection()
                if len(curSel):
                    curAts = curSel.findType(Atom)
                    if curAts != self.vf.allAtoms:
                        secondAts = curAts.inter(secondAts)
                        if not secondAts:  #NB inter returns empty set!
                            msg = 'no specified second group of atoms in current selection'
                            self.warningMsg(msg)
                            secondAts = AtomSet([])
                else:
                    msg = 'no current selection'
                    self.warningMsg(msg)
                    secondAts = AtomSet([])
        weight = self.ifd.entryByName['weight']['widget'].get()
        self.Close_cb()
        return self.doitWrapper(firstAts, secondAts, weight)


    def hideDSelector(self, event=None):
        e = self.ifd.entryByName['keyDSelector']
        if self.hideDSel.get():
            e['widget'].grid_forget()
        else:
            e['widget'].grid(e['gridcfg'])
        self.form.autoSize() 


    def hideASelector(self, event=None):
        e = self.ifd.entryByName['keyASelector']
        if self.hideASel.get():
            e['widget'].grid_forget()
        else:
            e['widget'].grid(e['gridcfg'])
        self.form.autoSize() 


    def __call__(self, first, second, weight=.1485, **kw):
        """None<-PyAD_calcVDWEnergies(first, second, weight=.1485)
first:one set of atoms
second:second set of atoms
weight: vdw weight to use
        """
        apply(self.doitWrapper, (first, second, weight), kw)


    def doit(self, first, second, weight):
        firstAts = self.vf.expandNodes(first)
        secondAts = self.vf.expandNodes(second)
        ms = self.ms = MolecularSystem()
        ms.add_entities(firstAts) 
        ms.add_entities(secondAts) 
        wmt = WeightedMultiTerm()
        wmt.set_molecular_system(ms)
        wmt.add_term( self.scorer, weight)
        result = wmt.get_score_array()
        #label each first atom by sum of its interaction energies 
        for i in range(len(first)):
            setattr(firstAts[i], self.prop, Numeric.add.reduce(result[i]))
        #label each second atom by sum of its vdw interaction energies
        swap_result = Numeric.swapaxes(result, 0,1)
        for i in range(len(swap_result)):
            setattr(secondAts[i], self.prop, Numeric.add.reduce(swap_result[i]))


PyADCalcVDWEnergiesGUI=CommandGUI()
PyADCalcVDWEnergiesGUI.addMenuCommand('AutoToolsBar', 'PyAutoDock', 'vdW', cascadeName = 'calculate intermolecular energies')



class PyAD4CalcVDWEnergies(PyADCalcVDWEnergies):
    """For each atom in one AtomSet, determine the vdw energy vs all the atoms in a second
    AtomSet"""

    def __init__(self, func=None):
        PyADCalcVDWEnergies.__init__(self, func)
        self.flag = self.flag | self.objArgOnly
        self.weight = .1560
        self.scorer = VanDerWaals()
        self.prop = 'ad4_vdw_energy'
        self.title = "For VanDerWaals Energies: Specify 'first' and 'second' group of atoms:"
        self.weightLabelTxt = "AD4 VDW weight"


    def __call__(self, first, second, weight=.1560, **kw):
        """None<-PyAD4_calcVDWEnergies(first, second, weight=.1560)
first:one set of atoms
second:second set of atoms
weight: vdw weight to use
        """
        apply(self.doitWrapper, (first, second, weight), kw)


PyAD4CalcVDWEnergiesGUI=CommandGUI()
PyAD4CalcVDWEnergiesGUI.addMenuCommand('AutoToolsBar', 'PyAutoDock', 'AD4_vdW', cascadeName = 'calculate intermolecular energies')



class PyADCalcHBONDEnergies(PyADCalcVDWEnergies):
    """For each atom in one AtomSet, determine the hydrogen bonding energy vs all the atoms in a second
    AtomSet"""


    def __init__(self, func=None):
        PyADCalcVDWEnergies.__init__(self, func)
        self.flag = self.flag | self.objArgOnly
        self.weight = .0656
        self.scorer = HydrogenBonding()
        self.prop = 'hb_energy'
        self.title = "For HydrogenBonding Energies: Specify 'first(receptor)' and 'second(ligand)' group of atoms:"
        self.weightLabelTxt = "HydrogenBonding weight"


    def __call__(self, first, second, weight=.0656, **kw):
        """None<-PyAD_calcHBONDEnergies(first, second, weight=.0656)
first:one set of atoms
second:second set of atoms
weight: hbonding weight to use
        """
        apply(self.doitWrapper, (first, second, weight), kw)


PyADCalcHBONDEnergiesGUI=CommandGUI()
PyADCalcHBONDEnergiesGUI.addMenuCommand('AutoToolsBar', 'PyAutoDock', 'hydrogen bonding', cascadeName = 'calculate intermolecular energies')



class PyAD4CalcHBONDEnergies(PyADCalcVDWEnergies):
    """For each atom in one AtomSet, determine the hydrogen bonding energy vs all the atoms in a second
    AtomSet"""


    def __init__(self, func=None):
        PyADCalcVDWEnergies.__init__(self, func)
        self.flag = self.flag | self.objArgOnly
        self.weight = .0974
        self.scorer = NewHydrogenBonding()
        self.prop = 'ad4_hb_energy'
        self.title = "For AD4 HydrogenBonding Energies: Specify 'first(receptor)' and 'second(ligand)' group of atoms:"
        self.weightLabelTxt = "AD4 HydrogenBonding weight"


    def __call__(self, first, second, weight=.0974, **kw):
        """None<-PyAD4_calcHBONDEnergies(first, second, weight=.0974)
first:one set of atoms
second:second set of atoms
weight: hbonding weight to use
        """
        apply(self.doitWrapper, (first, second, weight), kw)


    def doit(self, first, second, weight):
        firstAts = self.vf.expandNodes(first)
        secondAts = self.vf.expandNodes(second)
        ms = self.ms = MolecularSystem()
        ms.add_entities(firstAts) 
        ms.add_entities(secondAts) 
        wmt = WeightedMultiTerm()
        wmt.set_molecular_system(ms)
        wmt.add_term( self.scorer, weight)
        result = wmt.get_score_array()
        #label each first atom by sum of its vdw interaction energies 
        for i in range(len(first)):
            hbondmin = min(result[i])
            hbondmax = max(result[i])
            setattr(firstAts[i], self.prop, hbondmin+hbondmax)
            setattr(firstAts[i], 'hbondmin', hbondmin)
            setattr(firstAts[i], 'hbondmax', hbondmax)
            setattr(firstAts[i], 'sum_hb_energy', Numeric.add.reduce(result[i]))
        #label each second atom by sum of its vdw interaction energies
        swap_result = Numeric.swapaxes(result, 0,1)
        for i in range(len(swap_result)):
            hbondmin = min(swap_result[i])
            hbondmax = max(swap_result[i])
            setattr(secondAts[i], self.prop, hbondmin+hbondmax)
            setattr(secondAts[i], 'hbondmin', hbondmin)
            setattr(secondAts[i], 'hbondmax', hbondmax)
            setattr(secondAts[i], 'sum_hb_energy', Numeric.add.reduce(swap_result[i]))



PyAD4CalcHBONDEnergiesGUI=CommandGUI()
PyAD4CalcHBONDEnergiesGUI.addMenuCommand('AutoToolsBar', 'PyAutoDock', 'AD4_hydrogen bonding', cascadeName = 'calculate intermolecular energies')



class PyADCalcESTATEnergies(PyADCalcVDWEnergies):
    """For each atom in one AtomSet, determine the electrostatics energy vs all the atoms in a second
    AtomSet
    """

    def __init__(self, func=None):
        PyADCalcVDWEnergies.__init__(self, func)
        self.flag = self.flag | self.objArgOnly
        self.weight = .1146
        self.scorer = Electrostatics()
        self.prop = 'estat_energy'
        self.title = "For Electrostatics Energies: Specify 'first' and 'second' group of atoms:"
        self.weightLabelTxt = "Electrostatics weight"


    def __call__(self, first, second, weight=.1146, **kw):
        """None<-PyAD_calcESTATEnergies(first, second, weight=.1146)
first:one set of atoms
second:second set of atoms
weight: electrostatics weight to use
        """
        apply(self.doitWrapper, (first, second, weight), kw)


PyADCalcESTATEnergiesGUI=CommandGUI()
PyADCalcESTATEnergiesGUI.addMenuCommand('AutoToolsBar', 'PyAutoDock', 'electrostatics', cascadeName = 'calculate intermolecular energies')



class PyAD4CalcESTATEnergies(PyADCalcVDWEnergies):
    """For each atom in one AtomSet, determine the electrostatics energy vs all the atoms in a second
    AtomSet
    """


    def __init__(self, func=None):
        PyADCalcVDWEnergies.__init__(self, func)
        self.flag = self.flag | self.objArgOnly
        self.weight = .1465
        self.scorer = Electrostatics()
        self.prop = 'ad4_estat_energy'
        self.title = "For Electrostatics Energies: Specify 'first' and 'second' group of atoms:"
        self.weightLabelTxt = "AD4 Electrostatics weight"


    def __call__(self, first, second, weight=.1465, **kw):
        """None<-PyAD4_calcESTATEnergies(first, second, weight=.1465)
first:one set of atoms
second:second set of atoms
weight: electrostatics weight to use
        """
        apply(self.doitWrapper, (first, second, weight), kw)


PyAD4CalcESTATEnergiesGUI=CommandGUI()
PyAD4CalcESTATEnergiesGUI.addMenuCommand('AutoToolsBar', 'PyAutoDock', 'AD4_electrostatics', cascadeName = 'calculate intermolecular energies')



class PyADCalcDSOLVEnergies(PyADCalcVDWEnergies):
    """For each atom in one AtomSet, determine the hydrogen bonding energy vs all the atoms in a second
    AtomSet"""

    def __init__(self, func=None):
        PyADCalcVDWEnergies.__init__(self, func)
        self.flag = self.flag | self.objArgOnly
        self.weight = .1711
        self.scorer = Desolvation()
        self.prop = 'dsolv_energy'
        self.title = "For Desolvation Energies: Specify 'first(receptor)' and 'second(ligand)' group of atoms:"
        self.weightLabelTxt = "Desolvation weight"


    def __call__(self, first, second, weight=.1711, **kw):
        """None<-PyAD_calcDSOLVEnergies(first, second, DSOLVwt=.1711)
first:one set of atoms
second:second set of atoms
weight: dsolv weight to use
        """
        apply(self.doitWrapper, (first, second, weight), kw)


    def doit(self, first, second, weight):
        firstAts = self.vf.expandNodes(first)
        secondAts = self.vf.expandNodes(second)
        #KLUG here to fix AtSolPar and AtVol for protein
        #
        bothAts = firstAts + secondAts
        for a in bothAts:
            if a.parent.type + '_' + a.name in pep_aromList:
                a.autodock_element=='A'
                a.AtSolPar = .1027
            elif a.autodock_element=='A':
                a.AtSolPar = .1027
            elif a.autodock_element=='C':
                a.AtSolPar = .6844
            else:
                a.AtSolPar = 0.0
        ms = self.ms = MolecularSystem()
        ms.add_entities(firstAts) 
        ms.add_entities(secondAts) 
        wmt = WeightedMultiTerm()
        wmt.set_molecular_system(ms)
        wmt.add_term( self.scorer, weight)
        result = wmt.get_score_array()
        #fix O/H in the first row: arbitrarily set to solvation constant
        for i, n in zip(range(len(result)), result[0]):
            if n==.236 or n==.118:
                #print "1st row: zeroing ", i
                result[0][i]=0

        #label each first atom by its desolvation interaction energies 
        for i in range(len(first)):
            a = firstAts[i]
            if a.element=='O':
                setattr(a, self.prop, .236)   #check this
            elif a.element=='H':
                setattr(a, self.prop, .118)   #check this
            else:
                setattr(a, self.prop, Numeric.add.reduce(result[i]))
        #label each second atom by sum of its vdw interaction energies
        swap_result = Numeric.swapaxes(result, 0,1)
        #label each first atom by its desolvation interaction energies 
        for i in range(len(swap_result)):
            #secondAts[i].vdw_energy = Numeric.add.reduce(swap_result[i])
            a = secondAts[i]
            if a.element=='O':
                setattr(a, self.prop, .236)   #check this
            elif a.element=='H':
                setattr(a, self.prop, .118)   #check this
            else:
                setattr(a, self.prop, Numeric.add.reduce(swap_result[i]))
            #setattr(secondAts[i], self.prop, Numeric.add.reduce(swap_result[i]))


PyADCalcDSOLVEnergiesGUI=CommandGUI()
PyADCalcDSOLVEnergiesGUI.addMenuCommand('AutoToolsBar', 'PyAutoDock', 'desolvation', cascadeName = 'calculate intermolecular energies')



class PyAD4CalcDSOLVEnergies(PyADCalcVDWEnergies):
    """For each atom in one AtomSet, determine the hydrogen bonding energy vs all the atoms in a second
    AtomSet"""

    def __init__(self, func=None):
        PyADCalcVDWEnergies.__init__(self, func)
        self.flag = self.flag | self.objArgOnly
        self.weight = .1159
        self.scorer = NewDesolvation()
        self.prop = 'ad4_dsolv_energy'
        self.title = "For Desolvation Energies: Specify 'first' and 'second' group of atoms:"
        self.weightLabelTxt = "AD4 Desolvation weight"


    def __call__(self, first, second, weight=.1159, **kw):
        """None<-PyAD4_calcDSOLVEnergies(first, second, DSOLVwt=.1711)
first:one set of atoms
second:second set of atoms
weight: dsolv weight to use
        """
        apply(self.doitWrapper, (first, second, weight), kw)


PyAD4CalcDSOLVEnergiesGUI=CommandGUI()
PyAD4CalcDSOLVEnergiesGUI.addMenuCommand('AutoToolsBar', 'PyAutoDock', 
                            'AD4_desolvation', cascadeName = 'calculate intermolecular energies')



class PyADCalcAD3Energies(PyADCalcVDWEnergies):
    """For each atom in one AtomSet, determine the autodock3 energy vs all the atoms in a second
    AtomSet"""

    def __init__(self, func=None):
        PyADCalcVDWEnergies.__init__(self, func)
        self.flag = self.flag | self.objArgOnly
        self.weight = None
        self.weightLabel = None
        self.scorer = AutoDock305Scorer()
        self.prop = 'ad305_energy'
        self.title = "For AutoDock305 Energies: Specify 'first(receptor)' and 'second(ligand)' group of atoms:"
        self.weightLabelTxt = ""


    def onAddCmdToViewer(self):
        self.hideDSel = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        self.hideASel = Tkinter.IntVar(master=self.vf.GUI.ROOT)


    def buildForm(self):
        ifd = self.ifd = InputFormDescr(title = self.title)
        self.hideDSel = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        self.hideASel = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        self.useSelection = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        ifd.append({'name':'DkeyLab',
                    'widgetType':Tkinter.Label,
                    'text':'For First atoms:',
                    'gridcfg':{'sticky':'w','columnspan':3}})
        ifd.append({'name':'selDRB0',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Use all atoms',
                'variable': self.hideDSel,
                'value':1,
                'command':self.hideDSelector
                },
            'gridcfg':{'sticky':'w'}})
        ifd.append({'name':'selDRB1',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Set atoms to use',
                'variable': self.hideDSel,
                'value':0,
                'command':self.hideDSelector
                },
            'gridcfg':{'sticky':'w', 'row':-1, 'column':1}})
        ifd.append({'name': 'keyDSelector',
            'wtype':StringSelectorGUI,
            'widgetType':StringSelectorGUI,
            'wcfg':{ 'molSet': self.vf.Mols,
                    'vf': self.vf,
                    'all':1,
                    'crColor':(0.,1.,.2),
            },
            'gridcfg':{'sticky':'we', 'columnspan':2 }})
        #now the acceptors
        ifd.append({'name':'AkeyLab',
                    'widgetType':Tkinter.Label,
                    'text':'For Second atoms:',
                    'gridcfg':{'sticky':'w','columnspan':3}})
        ifd.append({'name':'selARB0',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Use all atoms',
                'variable': self.hideASel,
                'value':1,
                'command':self.hideASelector
                },
            'gridcfg':{'sticky':'w'}})
        ifd.append({'name':'selARB1',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Set atoms to use',
                'variable': self.hideASel,
                'value':0,
                'command':self.hideASelector
                },
            'gridcfg':{'sticky':'w', 'row':-1, 'column':1}})
        ifd.append({'name': 'keyASelector',
            'wtype':StringSelectorGUI,
            'widgetType':StringSelectorGUI,
            'wcfg':{ 'molSet': self.vf.Mols,
                    'vf': self.vf,
                    'all':1,
                    'crColor':(0.,1.,.2),
            },
            'gridcfg':{'sticky':'we', 'columnspan':2 }})
        ifd.append({'widgetType': Tkinter.Button,
            'text':'Ok',
            'wcfg':{'bd':6},
            'gridcfg':{'sticky':'ew', 'columnspan':2 },
            'command':self.Accept_cb})
        ifd.append({'widgetType': Tkinter.Button,
            'text':'Cancel',
            'wcfg':{'bd':6},
            'gridcfg':{'sticky':'ew', 'column':2,'row':-1},
            'command':self.Close_cb})
        self.form = self.vf.getUserInput(self.ifd, modal=0, blocking=0)
        self.hideDSelector()
        self.hideASelector()


    def Accept_cb(self, event=None):
        self.form.withdraw()
        firstNodesToCheck = self.ifd.entryByName['keyDSelector']['widget'].get()
        if not len(firstNodesToCheck):
            self.warningMsg('no first atoms specified')
            firstAts = AtomSet([])
        else:
            firstAts = firstNodesToCheck.findType(Atom)
            if self.useSelection.get():
                curSel = self.vf.getSelection()
                if len(curSel):
                    curAts = curSel.findType(Atom)
                    if curAts != self.vf.allAtoms:
                        firstAts = curAts.inter(firstAts)
                        if firstAts is None:
                            msg = 'no specified first atoms in current selection'
                            self.warningMsg(msg)
                            firstAts = AtomSet([])
                else:
                    msg = 'no current selection'
                    self.warningMsg(msg)
                    firstAts = AtomSet([])
        secondNodesToCheck = self.ifd.entryByName['keyASelector']['widget'].get()
        if not len(secondNodesToCheck):
            self.warningMsg('no second group of atoms specified')
            secondAts = AtomSet([])
            #return
        else:
            secondAts = secondNodesToCheck.findType(Atom)
            if self.useSelection.get():
                curSel = self.vf.getSelection()
                if len(curSel):
                    curAts = curSel.findType(Atom)
                    if curAts != self.vf.allAtoms:
                        secondAts = curAts.inter(secondAts)
                        if not secondAts:  #NB inter returns empty set!
                            msg = 'no specified second group of atoms in current selection'
                            self.warningMsg(msg)
                            secondAts = AtomSet([])
                else:
                    msg = 'no current selection'
                    self.warningMsg(msg)
                    secondAts = AtomSet([])
        self.Close_cb()
        return self.doitWrapper(firstAts, secondAts)


    def __call__(self, first, second, **kw):
        """None<-PyAD_calcAD3Energies(first, second)
first:one set of atoms
second:second set of atoms
        """
        apply(self.doitWrapper, (first, second,), kw)


    def doit(self, first, second ):
        firstAts = self.vf.expandNodes(first)
        secondAts = self.vf.expandNodes(second)
        bothAts = firstAts + secondAts
        for a in bothAts:
            if a.parent.type + '_' + a.name in pep_aromList:
                a.autodock_element=='A'
                a.AtSolPar = .1027
            elif a.autodock_element=='A':
                a.AtSolPar = .1027
            elif a.autodock_element=='C':
                a.AtSolPar = .6844
            else:
                a.AtSolPar = 0.0
        ms = self.ms = MolecularSystem()
        ms.add_entities(firstAts) 
        ms.add_entities(secondAts) 
        self.scorer.set_molecular_system(ms)
        score_array = self.scorer.get_score_array()
        self.scorer.labels_atoms_w_nrg(score_array)

PyADCalcAD3EnergiesGUI=CommandGUI()
PyADCalcAD3EnergiesGUI.addMenuCommand('AutoToolsBar', 'PyAutoDock', 'AutoDock305', cascadeName = 'calculate intermolecular energies')



class PyAD4CalcAD4Energies(PyADCalcAD3Energies):
    """For each atom in one AtomSet, determine the autodock4 energy vs all the atoms in a second
    AtomSet"""


    def __init__(self, func=None):
        PyADCalcAD3Energies.__init__(self, func)
        self.flag = self.flag | self.objArgOnly
        self.weight = None
        self.weightLabel = None
        self.scorer = AutoDock4Scorer()
        self.prop = 'ad4_energy'
        self.title = "For AutoDock4 Energies: Specify 'first(receptor)' and 'second(ligand)' group of atoms:"


    def __call__(self, first, second, **kw):
        """None<-PyAD4_calcAD4Energies(first, second)
first:one set of atoms
second:second set of atoms
        """
        apply(self.doitWrapper, (first, second,), kw)


    def doit(self, first, second):
        #FIX THIS: some scorers are symmetric and others are not
        firstAts = self.vf.expandNodes(first)
        secondAts = self.vf.expandNodes(second)
        ms = self.ms = MolecularSystem()
        ms.add_entities(firstAts) 
        ms.add_entities(secondAts) 
        self.scorer.set_molecular_system(ms)
        score_array = self.scorer.get_score_array()
        self.scorer.labels_atoms_w_nrg(score_array)
##         #NEED TO CORRECT hbond
##         #result = self.scorer.get_score_array()
##         sal = score_array_list = []
##         for t, w in self.scorer.terms:
##             sal.append(t.get_score_array()*w)
##         hbond_array = sal[1]
##         result = Numeric.add(sal[0], sal[2])
##         result = Numeric.add(result, sal[3])

##         #label each first atom by sum of its vdw interaction energies 
##         for i in range(len(first)):
##             #firstAts[i].vdw_energy = Numeric.add.reduce(result[i])
##             hbond_val =  min(hbond_array[i])+max(hbond_array[i])
##             setattr(firstAts[i], self.prop, Numeric.add.reduce(result[i])+hbond_val)
##         #label each second atom by sum of its vdw interaction energies
##         swap_result = Numeric.swapaxes(result, 0,1)
##         swap_hbond_array = Numeric.swapaxes(hbond_array, 0, 1)
##         for i in range(len(swap_result)):
##             #secondAts[i].vdw_energy = Numeric.add.reduce(swap_result[i])
##             hbond_val =  min(swap_hbond_array[i])+max(swap_hbond_array[i])
##             setattr(secondAts[i], self.prop, Numeric.add.reduce(swap_result[i])+hbond_val)


PyAD4CalcAD4EnergiesGUI=CommandGUI()
PyAD4CalcAD4EnergiesGUI.addMenuCommand('AutoToolsBar', 'PyAutoDock', 'AutoDock4', cascadeName = 'calculate intermolecular energies')



class PyAD4CalcInternalEnergies(PyAD4CalcAD4Energies):
    """For one AtomSet, determine the autodock4 energy of all the non-bond interactions:exclude 1-1, 1-2, 1-3, 1-4 and weedbonds in the same rigid piece.  This scorer uses AutoDock4Scorer for estat, vdw and dsolv energies PLUS a hbond1210 scorer for hbond"""

    def __init__(self, func=None):
        PyAD4CalcAD4Energies.__init__(self, func)
        self.flag = self.flag | self.objArgOnly
        iec = self.scorer = InternalEnergy(exclude_one_four=True, weed_bonds=True)
        ad4s = AutoDock4Scorer()
        self.weight = AutoDockTermWeights4() 
        #add the electrostatics term
        iec.add_term(ad4s.terms[0][0], self.weight.vdw_weight)
        #add a HydrogenBonding1210
        iec.add_term(NewHydrogenBonding12_10(), self.weight.hbond_weight)
        iec.add_term(ad4s.terms[2][0], self.weight.vdw_weight)
        iec.add_term(ad4s.terms[3][0], self.weight.dsolv_weight)
        self.prop = 'ad4_internal_energy'
        self.title = "For AD4 Internal Energies: Specify a group of atoms:"
        self.weightLabelTxt = ""


    def buildForm(self):
        ifd = self.ifd = InputFormDescr(title = self.title)
        self.hideDSel = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        self.hideASel = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        self.useSelection = Tkinter.IntVar(master=self.vf.GUI.ROOT)
        ifd.append({'name':'DkeyLab',
                    'widgetType':Tkinter.Label,
                    'text':'Atoms for internal energy calculation:',
                    'gridcfg':{'sticky':'w','columnspan':3}})
        ifd.append({'name':'selDRB0',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Use all atoms',
                'variable': self.hideDSel,
                'value':1,
                'command':self.hideDSelector
                },
            'gridcfg':{'sticky':'w'}})
        ifd.append({'name':'selDRB1',
            'widgetType':Tkinter.Radiobutton,
            'wcfg':{'text':'Set atoms to use',
                'variable': self.hideDSel,
                'value':0,
                'command':self.hideDSelector
                },
            'gridcfg':{'sticky':'w', 'row':-1, 'column':1}})
        ifd.append({'name': 'keyDSelector',
            'wtype':StringSelectorGUI,
            'widgetType':StringSelectorGUI,
            'wcfg':{ 'molSet': self.vf.Mols,
                    'vf': self.vf,
                    'all':1,
                    'crColor':(0.,1.,.2),
            },
            'gridcfg':{'sticky':'we', 'columnspan':2 }})
        ifd.append({'widgetType': Tkinter.Button,
            'text':'Ok',
            'wcfg':{'bd':6},
            'gridcfg':{'sticky':'ew', 'columnspan':2 },
            'command':self.Accept_cb})
        ifd.append({'widgetType': Tkinter.Button,
            'text':'Cancel',
            'wcfg':{'bd':6},
            'gridcfg':{'sticky':'ew', 'column':2,'row':-1},
            'command':self.Close_cb})
        self.form = self.vf.getUserInput(self.ifd, modal=0, blocking=0)
        self.hideDSelector()



    def Accept_cb(self, event=None):
        self.form.withdraw()
        firstNodesToCheck = self.ifd.entryByName['keyDSelector']['widget'].get()
        if not len(firstNodesToCheck):
            self.warningMsg('no first atoms specified')
            firstAts = AtomSet([])
        else:
            firstAts = firstNodesToCheck.findType(Atom)
            if self.useSelection.get():
                curSel = self.vf.getSelection()
                if len(curSel):
                    curAts = curSel.findType(Atom)
                    if curAts != self.vf.allAtoms:
                        firstAts = curAts.inter(firstAts)
                        if firstAts is None:
                            msg = 'no specified first atoms in current selection'
                            self.warningMsg(msg)
                            firstAts = AtomSet([])
                else:
                    msg = 'no current selection'
                    self.warningMsg(msg)
                    firstAts = AtomSet([])
        self.Close_cb()
        return self.doitWrapper(firstAts)


    def doit(self, first):
        firstAts = self.vf.expandNodes(first)
        ms = self.ms = MolecularSystem()
        ms.add_entities(firstAts) 
        self.scorer.set_molecular_system(ms)
        result = self.scorer.get_score_array()
        for i in range(len(first)):
            setattr(firstAts[i], self.prop, Numeric.add.reduce(result[i]))



PyAD4CalcInternalEnergiesGUI=CommandGUI()
PyAD4CalcInternalEnergiesGUI.addMenuCommand('AutoToolsBar', 'PyAutoDock', 'AD4', cascadeName = 'calculate internal energies') 



class PyAD305CalcInternalEnergies(PyAD4CalcInternalEnergies):
    """For one AtomSet, determine the autodock3 energy of all the non-bond interactions:exclude 1-1, 1-2, 1-3, 1-4 and weedbonds in the same rigid piece.  This scorer uses AutoDock4Scorer for estat, vdw and dsolv energies PLUS a hbond1210 scorer for hbond"""

    def __init__(self, func=None):
        PyAD4CalcInternalEnergies.__init__(self, func)
        self.flag = self.flag | self.objArgOnly
        iec = self.scorer = InternalEnergy(exclude_one_four=True, weed_bonds=True)
        ad3s = AutoDock305Scorer()
        self.weight = AutoDockTermWeights305() 
        #ad305 internal energy comprised of estat, vdw and hb1210
        #add the electrostatics term
        iec.add_term(ad3s.terms[0][0], self.weight.vdw_weight)
        #add a HydrogenBonding1210
        iec.add_term(HydrogenBonding12_10(), self.weight.hbond_weight)
        #add vanDerWaals
        iec.add_term(ad3s.terms[2][0], self.weight.vdw_weight)
        #ad305 does not include desolvation
        #iec.add_term(ad3s.terms[3][0], self.weight.dsolv_weight)
        self.prop = 'ad3_internal_energy'
        self.title = "For AD3 Internal Energies: Specify a group of atoms:"
        self.weightLabelTxt = ""


PyAD305CalcInternalEnergiesGUI=CommandGUI()
PyAD305CalcInternalEnergiesGUI.addMenuCommand('AutoToolsBar', 'PyAutoDock', 'AD305', cascadeName = 'calculate internal energies')





commandList = [
    
    {'name':'PyAD_calcClosestPairs','cmd':PyADCalcClosestPairs(),
        'gui':PyADCalcClosestPairsGUI},
    {'name':'PyAD_calcVDWEnergies','cmd':PyADCalcVDWEnergies(),
        'gui':PyADCalcVDWEnergiesGUI},
    {'name':'PyAD_calcHBONDEnergies','cmd':PyADCalcHBONDEnergies(),
        'gui':PyADCalcHBONDEnergiesGUI},
    {'name':'PyAD_calcESTATEnergies','cmd':PyADCalcESTATEnergies(),
        'gui':PyADCalcESTATEnergiesGUI},
    {'name':'PyAD_calcDSOLVEnergies','cmd':PyADCalcDSOLVEnergies(),
        'gui':PyADCalcDSOLVEnergiesGUI},
    {'name':'PyAD_calcAD3Energies','cmd':PyADCalcAD3Energies(),
        'gui':PyADCalcAD3EnergiesGUI},
    {'name':'PyAD4_calcVDWEnergies','cmd':PyAD4CalcVDWEnergies(),
        'gui':PyAD4CalcVDWEnergiesGUI},
    {'name':'PyAD4_calcHBONDEnergies','cmd':PyAD4CalcHBONDEnergies(),
        'gui':PyAD4CalcHBONDEnergiesGUI},
    {'name':'PyAD4_calcESTATEnergies','cmd':PyAD4CalcESTATEnergies(),
        'gui':PyAD4CalcESTATEnergiesGUI},
    {'name':'PyAD4_calcDSOLVEnergies','cmd':PyAD4CalcDSOLVEnergies(),
        'gui':PyAD4CalcDSOLVEnergiesGUI},
    {'name':'PyAD4_calcAD4Energies','cmd':PyAD4CalcAD4Energies(),
        'gui':PyAD4CalcAD4EnergiesGUI},
    {'name':'PyAD305_calcInternalEnergies','cmd':PyAD305CalcInternalEnergies(),
        'gui':PyAD305CalcInternalEnergiesGUI},
    {'name':'PyAD4_calcInternalEnergies','cmd':PyAD4CalcInternalEnergies(),
        'gui':PyAD4CalcInternalEnergiesGUI},
    ]

def initModule(vf):

    for dict in commandList:
        vf.addCommand(dict['cmd'],dict['name'],dict['gui'])


    if hasattr(vf, 'GUI'):
        for item in vf.GUI.menuBars['AutoToolsBar'].menubuttons.values():
            item.configure(background = 'tan')
            item.configure(underline = '-1')





#############################################################################
#
# Author: Ruth HUEY, Michel F. SANNER
#
# Copyright: M. Sanner TSRI 2000
#
#############################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/autodockHosts.py,v 1.2 2012/08/07 17:36:20 rhuey Exp $
#
# $Id: autodockHosts.py,v 1.2 2012/08/07 17:36:20 rhuey Exp $
#
#
#

import UserDict

class AutoDockHosts(UserDict.UserDict):

	def __init__(self, localHostDict):
		UserDict.UserDict.__init__(self)
		self.update(localHostDict)

	def buildEntry(self, host=None,agPath=None, adPath=None, vinaPath=None, qType='int', userSpecific=0):
		d={}
		d['host']=host
		d['autogrid']=agPath
		d['autodock']=adPath
		d['vina']=vinaPath
		d['queuetype']=qType
		d['userSpecific']=userSpecific
		return d

	def addHost(self,macroName, hostdict=None, **kw):
		host = kw['host']
		agPath = kw['autogrid']
		adPath = kw['autodock']
		vinaPath = kw['vina']
		qType = kw['queuetype']
		userSpecific = kw['userSpecific']
		if not hostdict:
			hostdict=self.buildEntry(host=host, agPath=agPath, adPath=adPath,
				qType=qType,userSpecific=userSpecific)
		self[macroName]=hostdict
		#nb: preexisting macroName entry is overwritten

	def saveHostFile(self, filename, whichOnes='all'):
		#will be in file called .adthosts.py
		#and consist of python code for dictionary called 'newhosts'
		fptr = open(filename, 'w')
		outstr = 'hostMacros={'
		#outstr = 'hosts={'
		fptr.write(outstr)
		#always write a localHost line
		#get the correct macroList here
		if whichOnes=='all':
			macroList=self.keys()
		elif whichOnes=='userSpecific':
			#get the one with userSpecific=1, only
			macroList=[]
			for item in self.items():
				if item[1]['userSpecific']: 
					macroList.append(item)
		else:
			macroList=[]
			for item in self.items():
				if not item[1]['userSpecific']: 
					macroList.append(item)
			#get the other ones...
		for i in range(len(macroList)):
			h=macroList[i][0]
			self.writeEntry(h,fptr)
			if i<len(macroList)-1:
				fptr.write('\t\t},\n')
			else:
				fptr.write('\t\t}\n')
		#close the whole thing
		outstr = '\t}\n'
		fptr.write(outstr)
		fptr.close()

	def writeEntry(self, macroName,fptr):
		outstr='\t\''+macroName+'\': {\n'
		#outstr='\t\''+hostName+'\': {\n'
		fptr.write(outstr)
		d = self[macroName]
		#d = self[hostName]
		klist = d.keys()
		for i in range(len(klist)):
			k=klist[i]
			if k=='userSpecific':
				outstr=	'\t\t\''+k + '\': '+ str(d[k])
			else:
				outstr=	'\t\t\''+k + '\': \''+ str(d[k])+'\''
			fptr.write(outstr)
			if i<len(klist)-1:
				outstr= ',\n'
			else:
				outstr= '\n'
			fptr.write(outstr)
		

	def loadHostFile(self, filename):
		newStuff=__import__(filename)
		self.update(filename.adhosts)
		#self.update(adhosts)

	



#############################################################################
#
# Author: Ruth HUEY, Stefano FORLI
#
# Copyright: A. Olson TSRI 2010
#
#############################################################################


# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/DlgFilters.py,v 1.6 2010/05/07 21:46:28 rhuey Exp $
#
# $Id: DlgFilters.py,v 1.6 2010/05/07 21:46:28 rhuey Exp $
#
#
#
#
#
#
#

"""
Assorted classes of filters for use in selecting subgroups of AutoDock Virtual Screening results...

filter_instance.filter(dlg) returns True/False 

"""
from string import strip
from numpy import oldnumeric as Numeric

class Filter:
    """
        Base class for object to screen dlg results

        input for filter: a dlg instance of the AutoDockTools 'Docking' class
        output: whether the dlg pass the criteria
    """
    def __init__(self, criteria='base_class'): 
        self.criteria = criteria
        self.keyline="BASECLASS"

    
    def get_lines(self, dlg):
        fptr = open(dlg)
        lines = fptr.readlines()
        fptr.close()
        for i in range(len(lines)):
            if lines[i].find(self.keyline)>-1:
                break
        return lines[i:]


    def filter(self, dlg):
        return True



class EnergyFilter(Filter):
    """
        object to evaluate a dlg results based on lowest energy of best docked result

        input: an AutoDockTools.Docking 'dlg' 
        output: whether the best energy of dlg is smaller than energy 
    """
    
    def __init__(self, energy=0):
        self.criteria = "BE"  
        Filter.__init__(self, self.criteria)
        self.energy = energy
        self.keyline= "CLUSTERING HISTOGRAM"

        
    def filter(self, dlg):
        lines = self.get_lines(dlg)
        bestcluster_line = lines[9]
        ll = bestcluster_line.split()
        best_energy = float(ll[2])
        mean_energy = float(ll[6])
        clu_size = int(ll[8])
        return best_energy<self.energy



class ClusterSizeFilter(Filter):
    """
        object to evaluate a dlg results based on size of the largest cluster  

        input: an AutoDockTools.Docking 'dlg' 
        output: whether the largest cluster in the dlg is at least as large as specified cluster_size
    """
    
    def __init__(self, cluster_size=0): 
        self.criteria = "LCsize"
        Filter.__init__(self, self.criteria)
        self.keyline= "CLUSTERING HISTOGRAM"
        self.cluster_size = cluster_size


    def filter(self, dlg):
        lines = self.get_lines(dlg)
        bestenergy_line = lines[9]
        ll = bestenergy_line.split()
        best_energy = float(ll[2])
        largest_clu_size = int(ll[8])
        #check for a larger cluster in lines which follow
        for nl in lines[9:]:
            nl_list = nl.split()
            if len(nl_list)>1 and nl_list[1]=='|':
                this_clu = float(nl_list[8]) 
                if this_clu>largest_clu_size:
                    largest_clu_size = this_clu
            elif nl_list[0]==nl[:-1]:
                break
        return largest_clu_size > self.cluster_size




class ClusterPercentageFilter(Filter):
    """
        object to evaluate a dlg results based on percentage of dlgs in the largest cluster  

        input: an AutoDockTools.Docking 'dlg' 
        output: whether the largest cluster in the dlg contains 
                at least the specified percentage of runs (docked results)
    """
    
    def __init__(self, percentage=0): 
        self.criteria = "LCpercentage"
        Filter.__init__(self, self.criteria)
        self.cluster_percentage = percentage
        self.keyline= "CLUSTERING HISTOGRAM"


    def filter(self, dlg):
        lines = self.get_lines(dlg)
        bestcluster_line = lines[9]
        ll = bestcluster_line.split()
        best_energy = float(ll[2])
        #mean_energy = float(ll[6])
        largest_clu_size = float(ll[8])
        total_clu = 0
        #have to add all the rest to this
        for nl in lines[9:]:
            nl_list = nl.split()
            if len(nl_list)>1 and nl_list[1]=='|':
                this_clu = float(nl_list[8]) 
                total_clu+= this_clu
                if this_clu>largest_clu_size:
                    largest_clu_size = this_clu
            elif nl_list[0]==nl[:-1]:
                break
        percent = 100.*(largest_clu_size/total_clu)
        #print "percent =", percent, " self.cluster_percentage=", self.cluster_percentage
        return percent >= self.cluster_percentage


        
class EnergyClusterSizeFilter(Filter):
    """
        object to evaluate a dlg results based on an energy and a size for the largest cluster
        
        input: an AutoDockTools.Docking 'dlg' 
        output: whether the lowest energy is at least as small as specified value 
                AND 
                whether largest cluster in the dlg is at least as large as specified cluster_size
    """

    def __init__(self, energy=0, cluster_size=0):
        self.criteria = "BE_Csize"  
        Filter.__init__(self, self.criteria)
        self.energyF = EnergyFilter(energy=energy)
        self.cluster_sizeF = ClusterSizeFilter(cluster_size=cluster_size)
        self.cluster_size = cluster_size
        self.keyline= "CLUSTERING HISTOGRAM"


    def filter(self, dlg):
        if not self.energyF.filter(dlg):
            return False
        # so it passed the energy
        if not self.cluster_sizeF.filter(dlg):
            return False
        # so it passed both the energy and cluster_size
        return True


class BestEnergyInLargestCluster(Filter):
    """
        object to screen dlg results based on whether the best energy conformation is in the largest cluster 

        input: an AutoDock dlg 
        output: whether the dlg passes these criteria
    """

    def __init__(self, energy=0):
        self.criteria = "BE_LC"  
        Filter.__init__(self, self.criteria)
        self.keyline= "CLUSTERING HISTOGRAM"


    def filter(self, dlg):
        best_energy = 100
        largest_clu_size = 0
        lines = self.get_lines(dlg)
        bestEnrg = lines[9].split()
        lowest_energy = float(bestEnrg[2])
        le_cluster_size = float(bestEnrg[8])
        #check for a larger cluster size
        ok = True
        for nl in lines[10:]:
            #'_' marks end of cluster output
            if nl[0]=='_': 
                break
            nl_list = nl.split()
            this_clu_size = float(nl_list[8]) 
            if this_clu_size>le_cluster_size:
                largest_clu_size = this_clu_size
                ok = False
                break
        return ok



class EnergyPlusBestEnergyInLargestCluster(Filter):
    """
        object to screen dlg results based on the magnitude of best energy AND 
                                              whether that conf is in the largest cluster 
        input: an AutoDock dlg 
        output: whether the dlg passes these criteria
    """

    def __init__(self, energy=0):
        self.criteria = "BE_LC"  
        Filter.__init__(self, self.criteria)
        self.energyF = EnergyFilter(energy=energy)
        self.be_lcF = BestEnergyInLargestCluster(energy)
        self.keyline= "CLUSTERING HISTOGRAM"


    def filter(self, dlg):
        #check energy first
        if not self.energyF.filter(dlg):
            return False
        #now check whether best energy is in largest cluster
        if not self.be_lcF.filter(dlg):
            return False
        return  True



class LigandEfficiencyFilter(Filter):
    """
        object to screen dlg results based on the 'ligand efficiency' measure which is the 
                            best energy divided by the number of heavy atoms in the ligand

        input: an AutoDock dlg 
        output: whether the dlg passes the criteria
    """

    def __init__(self, ligand_efficiency=0, verbose=False):
        self.criteria = "Ligand_efficiency"  
        Filter.__init__(self, self.criteria)
        self.ligand_efficiency = ligand_efficiency
        self.keyline= "Total number of atoms found"
        self.infoline= "CLUSTERING HISTOGRAM"
        self.h_map_line = ', atom type "HD", grid map index ='
        self.number_of_hydrogens = 0
        self.verbose = verbose

    
    def get_lines(self, dlg):
        fptr = open(dlg)
        lines = fptr.readlines()
        fptr.close()
        atom_numbers = []
        found_index = False
        hmap = -1
        for i in range(len(lines)):
            cur_line = lines[i]
            if not found_index and cur_line.find(self.h_map_line)>-1:
                hmap = int(strip(cur_line.split()[-1]))
                if self.verbose: print "hmap=", hmap
                found_index = 1
            if lines[i].find(self.keyline)>-1:
                break
        for j in range(len(lines[i:])):
            if lines[j].find("Number of atoms with atom type %d ="%(hmap))>-1:
                self.number_of_hydrogens = int(strip(lines[j].split()[-1]))
                if self.verbose: print "found %d hydrogens" %(self.number_of_hydrogens)
        return lines[i:]


    def filter(self, dlg):
        lines = self.get_lines(dlg)
        #only count heavy atoms so need to remove the hydrogens...
        num_lig_atoms = int(lines[0].split()[-2]) - self.number_of_hydrogens
        for i in range(len(lines)):
            if lines[i].find(self.infoline)>-1:
                break
        ll = lines[i+9].split()
        best_energy = float(ll[6])
        lig_eff = float(round(best_energy/num_lig_atoms, 4))
        if self.verbose: print " ligand efficiency = ", lig_eff
        return lig_eff<=self.ligand_efficiency



class InteractionFilter(Filter):       
    """
        class for object to screen dlg results based on presence of specific ligand-receptor interactions
            initialized with a file containing the residue(s) of interest in the receptor

        input: an AutoDock dlg 
        output: whether there are any contacts between a docked pose in the  dlg and the atoms in the 'receptor_file'
    """

    def __init__(self, receptor_file):
        from PyBabel.babelElements import babel_elements
        self.babel_elements = babel_elements
        self.criteria = "Interactions"  
        Filter.__init__(self, self.criteria)
        self.receptor_file = receptor_file
        rptr = open(receptor_file)
        rec_lines = rptr.readlines()
        rptr.close()
        rec_coords =[]
        rec_vdw_rad =[]
        for ll in rec_lines:
            if ll.find("HETATM")>-1 or ll.find("ATOM")>-1:
                rec_coords.append([float(ll[30:38]), float(ll[38:46]),float(ll[46:54])])
                be_key = strip(ll[76:])
                if be_key=='A':
                    be_key = 'C'
                elif be_key=='OA':
                    be_key = 'O'
                elif be_key=='NA':
                    be_key = 'N'
                elif be_key=='SA':
                    be_key = 'S'
                elif be_key=='HD':
                    be_key = 'H'
                rec_vdw_rad.append(babel_elements[be_key]['vdw_rad'])
        # lenC and bigC are about the receptor...
        self.lenC = len(rec_vdw_rad)
        lenC = self.lenC
        #len(rec_vdw_rad)
        #1844
        #len(rec_coords)
        #1844
        checkRadii = Numeric.array(rec_vdw_rad, 'f')
        self.bigRC = Numeric.resize(checkRadii, (lenC, lenC)) #receptor info
        self.bigC = Numeric.resize(rec_coords, (lenC,lenC,3))
        ###bigR = bigRC[:lenK]
        

    def setup_ligand(self, lig_lines):
        lig_coords = []
        first_lig = False
        if not hasattr(self, 'lig_vdw_rad'):
            self.lig_vdw_rad = []
            first_lig = True
        for ll in lig_lines:
            #THESE LINES START WITH 'DOCKED: ' so indices +8
            if ll.find("HETATM")>-1 or ll.find("ATOM")>-1:
                lig_coords.append([float(ll[38:46]), float(ll[46:54]),float(ll[54:62])])
                #lig_coords.append([float(ll[30:38]), float(ll[38:46]),float(ll[46:54])])
                if first_lig: 
                    #indices  +8
                    be_key = strip(ll[84:])
                    if be_key=='A':
                        be_key = 'C'
                    elif be_key=='OA':
                        be_key = 'O'
                    elif be_key=='NA':
                        be_key = 'N'
                    elif be_key=='SA':
                        be_key = 'S'
                    elif be_key=='HD':
                        be_key = 'H'
                    self.lig_vdw_rad.append(self.babel_elements[be_key]['vdw_rad'])
        self.lig_coords = lig_coords
        self.smallM = Numeric.array(lig_coords, 'f')
        #setup variables for ligand arrays
        if first_lig:
            self.lenK=len(lig_coords)
            self.keyRadii = Numeric.array(self.lig_vdw_rad, 'f')
            self.keyRadii.shape = (self.lenK,1)
        self.smallM.shape = (self.lenK,1,3)

        

    def filter(self, dlg):
        fptr = open(dlg)
        dlg_lines = fptr.readlines()
        fptr.close()
        #STEP 1:accumulate lines of various poses
        model_lines = []
        #keep all of them
        all_models = []
        in_model = False
        for ll in dlg_lines:
            if ll.find("DOCKED:")==0:
                #check for a new model
                if ll.find("DOCKED: MODEL")==0:
                    model_lines = []
                in_model = True
                model_lines.append(ll)
            if ll.find("_")==0 and in_model:
                all_models.append(model_lines)
                model_lines = []
                in_model = False
        #initialize this ligand 
        # loop over the models:
        for model_lines in all_models:
            self.setup_ligand(model_lines)
            bigR = self.bigRC[:self.lenK]
            bigM = self.bigC[:self.lenK]
            cutoff = bigR + self.keyRadii
            d = bigM - self.smallM
            dSQ = d*d
            dSQMAT = Numeric.sum(dSQ,2)
            cutoffSQMAT = cutoff*cutoff
            ansMat = Numeric.logical_and(Numeric.less(dSQMAT, cutoffSQMAT),Numeric.not_equal(dSQMAT, 0.))
            rowIndices = Numeric.nonzero(Numeric.sum(ansMat,1))
            num_contacts = 0
            for ind in rowIndices:
                for j in ansMat[ind]: 
                    if j: num_contacts+=1
            if num_contacts > 0:
                break 
        return num_contacts

